/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.security.core.server;

import java.math.BigInteger;
import java.security.Principal;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import javax.security.auth.Subject;
import javax.security.auth.x500.X500Principal;

import org.apache.axis2.context.MessageContext;
import org.apache.log4j.Logger;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSSecurityEngineResult;
import org.apache.ws.security.WSSecurityException;
import org.apache.ws.security.components.crypto.Crypto;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.apache.ws.security.handler.WSHandlerResult;
import org.ogf.graap.wsag.security.core.SecurityConstants;
import org.ogf.graap.wsag.security.core.SecurityUtils;
import org.ogf.graap.wsag.server.api.WsagMessageContext;
import org.ogf.graap.wsag.server.engine.WsagEngine;
import org.w3c.dom.Element;

/**
 * The WS-Security handler checks the WS-Security processing results and populates the
 * {@link WsagEngine#getWsagMessageContext()} with the client subject, the client certificate and the client
 * certificate chain. In case security is disabled and anonymous access to the server is permitted the handler
 * initializes an
 * 
 * @author Oliver Waeldrich
 * 
 */
public class WSSecurityHandler implements ServerSecurityHandler
{

    private static final Logger LOG = Logger.getLogger( WSSecurityHandler.class );

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.wsrf.handler.AxisHandler#handleRequest(org.apache.axiom.om.OMElement)
     */
    public void handleRequest( Element request ) throws Exception
    {
        MessageContext messageContext = MessageContext.getCurrentMessageContext();

        Crypto crypto = SecurityUtils.getCryptoFromLoginContext( WsagEngine.getLoginContext() );

        if ( crypto != null )
        {
            processSecurityHeader( crypto, messageContext );
        }
        else
        {
            LOG.error( "Could not process security headers. Reason: server crypto not found." );
        }
    }

    /**
     * {@inheritDoc}
     */
    public void handleResponse( Element response ) throws Exception
    {
        // nothing to do
    }

    @SuppressWarnings( { "unchecked" } )
    private void processSecurityHeader( Crypto crypto, MessageContext messageContext )
        throws WSSecurityException
    {
        WsagMessageContext context = WsagEngine.getWsagMessageContext();
        Vector<WSHandlerResult> handlerResults =
            (Vector<WSHandlerResult>) messageContext.getProperty( WSHandlerConstants.RECV_RESULTS );

        X509Certificate userCertificate = null;
        // X509Certificate[] bstCertificates = null;

        for ( int i = 0; i < handlerResults.size(); i++ )
        {
            WSHandlerResult currentResult = handlerResults.get( i );

            Vector<WSSecurityEngineResult> wsSecEngineResults = currentResult.getResults();
            for ( int j = 0; j < wsSecEngineResults.size(); j++ )
            {
                WSSecurityEngineResult wser = wsSecEngineResults.get( j );
                if ( ( (Integer) wser.get( WSSecurityEngineResult.TAG_ACTION ) ).intValue() == WSConstants.SIGN
                    && wser.get( WSSecurityEngineResult.TAG_X509_CERTIFICATE ) != null )
                {

                    userCertificate =
                        (X509Certificate) wser.get( WSSecurityEngineResult.TAG_X509_CERTIFICATE );

                    if ( LOG.isTraceEnabled() )
                    {
                        String issuerName =
                            ( (Principal) wser.get( WSSecurityEngineResult.TAG_PRINCIPAL ) ).getName();
                        LOG.trace( "Found WS-Security engine result. Message signed by " + issuerName );
                    }

                    context.put( SecurityConstants.X509_CLIENT_CERTIFICATE, userCertificate );

                    // Construct the client certificate chain from the request
                    context.put( SecurityConstants.X509_CLIENT_CERTIFICATE_CHAIN,
                                 createClientCertificateChain( crypto, userCertificate ) );
                }
            }
        }

        //
        // add the security subject for authentication
        //
        // The subject is created based on the certificated that was used to sign the message.
        // If no certificate is provided and anonymous usage is allowed an anonymous subject is
        // created. Otherwise, an exception is raised.
        //
        if ( userCertificate != null )
        {
            Subject subject = new Subject();
            subject.getPrincipals().add( userCertificate.getSubjectX500Principal() );
            subject.getPrincipals().add( userCertificate.getIssuerX500Principal() );
            subject.getPublicCredentials().add( userCertificate );
            
            WsagEngine.getWsagMessageContext().put( SecurityConstants.AUTHENTICATED_USER, subject );
        }
        else if ( WsagEngine.isAllowAnonymousAccess() )
        {
            Subject subject = new Subject();
            subject.getPrincipals().add( new X500Principal( "anonymous" ) );
            subject.getPublicCredentials().add( userCertificate );
            WsagEngine.getWsagMessageContext().put( SecurityConstants.AUTHENTICATED_USER, subject );
        }
        else
        {
            throw new WSSecurityException( "The user was not authenticated." );
        }
    }

    /**
     * Creates the certificate chain for the authenticated client. This method contains code from the
     * <code>org.apache.ws.axis.security.WSDoAllReceiver.verifyTrust()</code> method.
     * 
     * @param crypto
     *            the server crypto
     * @param certificate
     *            the client certificate
     * @return the client certificate chain
     * 
     * @throws WSSecurityException
     *             indicates an error while creating the certificate chain
     */
    private X509Certificate[] createClientCertificateChain( Crypto crypto, X509Certificate certificate )
        throws WSSecurityException
    {
        X509Certificate[] result;

        // First step: Search the keystore for the transmitted certificate
        String subjectString = certificate.getSubjectDN().getName();
        String issuerString = certificate.getIssuerDN().getName();
        BigInteger issuerSerial = certificate.getSerialNumber();

        String alias = crypto.getAliasForX509Cert( issuerString, issuerSerial );

        if ( alias != null )
        {
            // Retrieve the certificate for the alias from the keystore
            try
            {
                result = crypto.getCertificates( alias );

                // If certificates have been found, the certificates must be compared
                // to ensure against phony DNs (compare encoded form including signature)
                if ( result != null && result.length > 0 && certificate.equals( result[0] ) )
                {
                    return result;
                }
                result = null;
            }
            catch ( WSSecurityException ex )
            {
                throw new WSSecurityException( getClass().getName()
                    + ": Could not get certificates for alias " + alias, ex );
            }
        }

        // SECOND step: Search for the issuer of the transmitted certificate in the keystore
        String[] aliases = null;
        try
        {
            aliases = crypto.getAliasesForDN( issuerString );
        }
        catch ( WSSecurityException ex )
        {
            throw new WSSecurityException( getClass().getName()
                + ": Could not get alias for certificate with " + issuerString );
        }

        // If the alias has not been found, the issuer is not in the keystore
        // As a direct result, do not trust the transmitted certificate
        if ( aliases == null || aliases.length < 1 )
        {
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "No aliases found in keystore for issuer " + issuerString + " of certificate for "
                    + subjectString + "." );
            }

            return null;
        }

        // THIRD step
        // Check the certificate trust path for every alias of the issuer found in the keystore
        for ( int i = 0; i < aliases.length; i++ )
        {
            alias = aliases[i];

            if ( LOG.isTraceEnabled() )
            {
                LOG.trace( "Preparing to validate certificate path with alias " + alias + " for issuer "
                    + issuerString + "." );
            }

            // Retrieve the certificate(s) for the alias from the keystore
            X509Certificate[] certs;
            try
            {
                certs = crypto.getCertificates( alias );
            }
            catch ( WSSecurityException ex )
            {
                throw new WSSecurityException( getClass().getName()
                    + ": Could not get certificates for alias " + alias, ex );
            }

            // If no certificates have been found, there has to be an error:
            // The keystore can find an alias but no certificate(s)
            if ( certs == null | certs.length < 1 )
            {
                throw new WSSecurityException( getClass().getName()
                    + ": Could not get certificates for alias " + alias );
            }

            // Use the validation method from the crypto to check whether the subjects certificate was really
            // signed by
            // the issuer stated in the certificate
            try
            {
                if ( crypto.validateCertPath( certs ) )
                {
                    if ( LOG.isTraceEnabled() )
                    {
                        LOG.trace( getClass().getName()
                            + ": Certificate path has been verified for certificate with subject "
                            + subjectString );
                    }

                    // ordered users cert first, root ca last, see keystore
                    List<X509Certificate> resultCerts = new Vector<X509Certificate>();
                    resultCerts.add( certificate );
                    resultCerts.addAll( Arrays.asList( certs ) );
                    return resultCerts.toArray( new X509Certificate[resultCerts.size()] );
                }
            }
            catch ( WSSecurityException ex )
            {
                throw new WSSecurityException( getClass().getName()
                    + ": Certificate path verification failed for certificate with subject " + subjectString,
                                               ex );
            }
        }

        if ( LOG.isInfoEnabled() )
        {
            LOG.info( getClass().getName() + ": Could not retrieve client certificate chain for subject "
                + subjectString );
        }

        return null;
    }

}
