/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf;

import java.util.Set;

import javax.security.auth.x500.X500Principal;
import javax.xml.namespace.QName;

import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.impl.SimpleWsResource;
import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.server.engine.WsagEngine;
import org.ogf.graap.wsag4j.types.engine.ServerIdentityDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3.x2005.x08.addressing.MetadataType;
import org.w3c.dom.Node;

/**
 * Abstract implementation of a WSRF resource. This class implements basic mechanisms to use XmlBeans EPRs
 * with a WSRF resource. Moreover, the server identity is added automatically to EPR meta data when the WSRF
 * resource is initialized. This is done for convenience when clients want to delegate trust to a service that
 * is implemented as WSRF resource.
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AbstractWsResource extends SimpleWsResource
{

    private boolean initialized = false;

    /**
     * 
     * @return the QName of the implemented port type
     */
    public abstract QName getInterfaceName();

    /**
     * Sets the EPR for this WS-Resource and adds the server identity to the meta data of the EPR.
     * 
     * @param epr
     *            the endpoint reference of this resource
     */
    public void setEndpointReference( EndpointReferenceType epr )
    {
        setEndpointReference( XmlUtils.convertEndpointToMuseEPR( epr ) );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.apache.muse.ws.resource.impl.SimpleWsResource#initialize()
     */
    public void initialize() throws SoapFault
    {
        //
        // first add the server identity to the EPR
        //
        EndpointReferenceType epr = XmlUtils.convertMuseEPRToEndpoint( getEndpointReference() );
        addServerIdentityToEPR( epr );
        setEndpointReference( XmlUtils.convertEndpointToMuseEPR( epr ) );

        //
        // now initialize the WS resource
        //
        super.initialize();

        initialized = true;
    }

    private void addServerIdentityToEPR( EndpointReferenceType epr )
    {
        if ( !epr.isSetMetadata() )
        {
            epr.addNewMetadata();
        }

        MetadataType meta = epr.getMetadata();
        XmlObject[] identities = meta.selectChildren( ServerIdentityDocument.type.getDocumentElementName() );
        for ( int i = 0; i < identities.length; i++ )
        {
            meta.getDomNode().removeChild( identities[i].getDomNode() );
        }

        Set<X500Principal> principals =
            WsagEngine.getLoginContext().getSubject().getPrincipals( X500Principal.class );
        if ( !principals.isEmpty() )
        {
            X500Principal principal = principals.iterator().next();

            ServerIdentityDocument id = ServerIdentityDocument.Factory.newInstance();
            id.setServerIdentity( principal.getName() );
            Node imported =
                meta.getDomNode().getOwnerDocument().importNode( id.getDomNode().getFirstChild(), true );
            meta.getDomNode().appendChild( imported );
        }
    }

    /**
     * Returns true only after the resource initialization process is complete. By default an additional
     * schema validation is performed where the properties of the agreement are accessed.
     * 
     * @return the initialized
     */
    public boolean isInitialized()
    {
        return initialized;
    }
}
