/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.bootstrap;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CountDownLatch;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXBuilder;
import org.apache.axiom.soap.SOAP12Constants;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFaultCode;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.addressing.AddressingHelper;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.deployment.WarBasedAxisConfigurator;
import org.apache.axis2.description.AxisBindingMessage;
import org.apache.axis2.description.AxisBindingOperation;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.description.TransportOutDescription;
import org.apache.axis2.description.WSDL2Constants;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.engine.AxisEngine;
import org.apache.axis2.engine.Handler.InvocationResponse;
import org.apache.axis2.engine.ListenerManager;
import org.apache.axis2.transport.RequestResponseTransport;
import org.apache.axis2.transport.TransportListener;
import org.apache.axis2.transport.TransportUtils;
import org.apache.axis2.transport.http.AxisServletListener;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.transport.http.HTTPTransportUtils;
import org.apache.axis2.transport.http.ListingAgent;
import org.apache.axis2.transport.http.ServletBasedOutTransportInfo;
import org.apache.axis2.transport.http.TransportHeaders;
import org.apache.axis2.transport.http.util.QueryStringParser;
import org.apache.axis2.transport.http.util.RESTUtil;
import org.apache.axis2.util.JavaUtils;
import org.apache.axis2.util.MessageContextBuilder;
import org.apache.log4j.Logger;

/**
 * This is essentially a copy of the Axis2 Servlet. It includes a patch in the initialization method to
 * prevent the registration of a shutdown hook by the ListenerManager ({@link #init(ServletConfig)}), which
 * causes a memory leak when re-deploying in tomcat.
 * 
 * When upgrading to a new version, this file MUST be upgraded too. (probably the shutdown hook can be
 * prevented in future by configuration)
 * 
 * @author Oliver Waeldrich
 * 
 */
// CHECKSTYLE:OFF
@SuppressWarnings( "serial" )
public class Bootstrap extends HttpServlet
{

    private static final Logger LOG = Logger.getLogger( Bootstrap.class );

    /**
     * Configuration Context
     */
    public static final String CONFIGURATION_CONTEXT = "CONFIGURATION_CONTEXT";

    /**
     * Session ID
     */
    public static final String SESSION_ID = "SessionId";

    private static final Set<String> METADATA_QUERY_PARAM_NAMES =
        new HashSet<String>( Arrays.asList( "wsdl2", "wsdl", "xsd", "policy" ) );

    /**
     * Configuration Context
     */
    protected transient ConfigurationContext configContext;

    /**
     * Axis Configuration
     */
    protected transient AxisConfiguration axisConfiguration;

    /**
     * Servlet Configuration
     */
    protected transient ServletConfig servletConfig;

    /**
     * The Listing Agent.
     */
    protected transient ListingAgent agent;

    /**
     * Context Root.
     */
    protected transient String contextRoot = null;

    /**
     * disable REST
     */
    protected boolean disableREST = false;

    private static final String LIST_SERVICES_SUFFIX = "/services/listServices";

    private static final String LIST_FAULTY_SERVICES_SUFFIX = "/services/ListFaultyServices";

    private boolean closeReader = true;

    private static final int BUFFER_SIZE = 1024 * 8;

    private boolean initCalled = false;

    private transient AxisServletListener httpListener;

    private transient AxisServletListener httpsListener;

    /**
     * Implementation of POST interface
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @throws ServletException
     *             an servlet error
     * @throws IOException
     *             an i/o error
     */
    @SuppressWarnings( "unused" )
    protected void doPost( HttpServletRequest request, HttpServletResponse response )
        throws ServletException, IOException
    {

        LOG.trace( "entering axis servlet post call" );

        // set the initial buffer for a larger value
        response.setBufferSize( BUFFER_SIZE );

        preprocessRequest( request );

        MessageContext msgContext;
        OutputStream out = response.getOutputStream();
        String contentType = request.getContentType();
        if ( !HTTPTransportUtils.isRESTRequest( contentType ) )
        {
            msgContext = createMessageContext( request, response );
            msgContext.setProperty( Constants.Configuration.CONTENT_TYPE, contentType );
            try
            {
                // adding ServletContext into msgContext;
                String url = request.getRequestURL().toString();

                OutputStream bufferedOut = new BufferedOutputStream( out );

                InvocationResponse pi =
                    HTTPTransportUtils.processHTTPPostRequest( msgContext,
                                                               new BufferedInputStream(
                                                                                        request.getInputStream() ),
                                                               bufferedOut,
                                                               contentType,
                                                               request.getHeader( HTTPConstants.HEADER_SOAP_ACTION ),
                                                               url );

                Boolean holdResponse =
                    (Boolean) msgContext.getProperty( RequestResponseTransport.HOLD_RESPONSE );

                RequestResponseTransport transport =
                    (RequestResponseTransport) msgContext.getProperty( RequestResponseTransport.TRANSPORT_CONTROL );

                if ( pi.equals( InvocationResponse.SUSPEND )
                    || ( holdResponse != null && Boolean.TRUE.equals( holdResponse ) ) )
                {
                    transport.awaitResponse();
                }
                response.setContentType( "text/xml; charset="
                    + msgContext.getProperty( Constants.Configuration.CHARACTER_SET_ENCODING ) );
                // if data has not been sent back and this is not a signal response
                if ( !TransportUtils.isResponseWritten( msgContext )
                    && ( transport.getStatus() != RequestResponseTransport.RequestResponseTransportStatus.SIGNALLED ) )
                {
                    response.setStatus( HttpServletResponse.SC_ACCEPTED );
                }

                // Make sure that no data remains in the BufferedOutputStream even if the message
                // formatter doesn't call flush
                bufferedOut.flush();

            }
            catch ( AxisFault e )
            {
                setResponseState( msgContext, response );
                LOG.debug( "catch axis fault", e );
                if ( msgContext != null )
                {
                    processAxisFault( msgContext, response, out, e );
                }
                else
                {
                    throw new ServletException( e );
                }
            }
            catch ( Throwable t )
            {
                LOG.error( t.getMessage(), t );
                try
                {
                    // If the fault is not going along the back channel we should be 202ing
                    if ( AddressingHelper.isFaultRedirected( msgContext ) )
                    {
                        response.setStatus( HttpServletResponse.SC_ACCEPTED );
                    }
                    else
                    {
                        response.setStatus( HttpServletResponse.SC_INTERNAL_SERVER_ERROR );

                        AxisBindingOperation axisBindingOperation =
                            (AxisBindingOperation) msgContext.getProperty( Constants.AXIS_BINDING_OPERATION );
                        if ( axisBindingOperation != null )
                        {
                            String faultName = (String) msgContext.getProperty( Constants.FAULT_NAME );
                            AxisBindingMessage axisBindingMessage = axisBindingOperation.getFault( faultName );
                            if ( axisBindingMessage != null )
                            {
                                Integer code =
                                    (Integer) axisBindingMessage.getProperty( WSDL2Constants.ATTR_WHTTP_CODE );
                                if ( code != null )
                                {
                                    response.setStatus( code.intValue() );
                                }
                            }
                        }
                    }
                    handleFault( msgContext, out, new AxisFault( t.toString(), t ) );
                }
                catch ( AxisFault e2 )
                {
                    LOG.info( e2 );
                    throw new ServletException( e2 );
                }
            }
            finally
            {
                LOG.trace( "leaving axis servlet post call" );

                closeStaxBuilder( msgContext );
                TransportUtils.deleteAttachments( msgContext );
            }
        }
        else
        {
            if ( !disableREST )
            {
                RestRequestProcessor processor =
                    new RestRequestProcessor( Constants.Configuration.HTTP_METHOD_POST, request, response );
                processor.processXMLRequest();
            }
            else
            {
                showRestDisabledErrorMessage( response );
            }
        }
    }

    /**
     * Implementation for GET interface
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @throws ServletException
     *             an servlet error
     * @throws IOException
     *             an i/o error
     */

    protected void doGet( HttpServletRequest request, HttpServletResponse response )
        throws ServletException, IOException
    {

        preprocessRequest( request );

        // this method is also used to serve for the listServices request.

        String requestURI = request.getRequestURI();
        String query = request.getQueryString();

        // There can be three different request coming to this.
        // 1. wsdl, wsdl2 and xsd requests
        // 2. list services requests
        // 3. REST requests.
        if ( ( query != null ) && new QueryStringParser( query ).search( METADATA_QUERY_PARAM_NAMES ) )
        {
            // handling meta data exchange stuff
            agent.processListService( request, response );
        }
        else if ( requestURI.endsWith( ".xsd" ) || requestURI.endsWith( ".wsdl" ) )
        {
            agent.processExplicitSchemaAndWSDL( request, response );
        }
        else if ( requestURI.endsWith( LIST_SERVICES_SUFFIX )
            || requestURI.endsWith( LIST_FAULTY_SERVICES_SUFFIX ) )
        {
            // handling list services request
            try
            {
                agent.handle( request, response );
            }
            catch ( Exception e )
            {
                throw new ServletException( e );
            }
        }
        else if ( !disableREST )
        {
            new RestRequestProcessor( Constants.Configuration.HTTP_METHOD_GET, request, response ).processURLRequest();
        }
        else
        {
            showRestDisabledErrorMessage( response );
        }
    }

    /**
     * Implementation of DELETE interface
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @throws ServletException
     *             servlet error
     * @throws IOException
     *             I/O error
     */

    protected void doDelete( HttpServletRequest request, HttpServletResponse response )
        throws ServletException, IOException
    {

        preprocessRequest( request );
        // this method is also used to serve for the listServices request.
        if ( !disableREST )
        {
            RestRequestProcessor processor =
                new RestRequestProcessor( Constants.Configuration.HTTP_METHOD_DELETE, request, response );
            processor.processURLRequest();
        }
        else
        {
            showRestDisabledErrorMessage( response );
        }
    }

    /**
     * Implementation of PUT interface
     * 
     * @param request
     *            the request
     * @param response
     *            the response
     * @throws ServletException
     *             an servlet error
     * @throws IOException
     *             an i/o error
     */
    protected void doPut( HttpServletRequest request, HttpServletResponse response )
        throws ServletException, IOException
    {

        preprocessRequest( request );
        // this method is also used to serve for the listServices request.
        if ( !disableREST )
        {
            new RestRequestProcessor( Constants.Configuration.HTTP_METHOD_PUT, request, response ).processXMLRequest();
        }
        else
        {
            showRestDisabledErrorMessage( response );
        }
    }

    /**
     * Private method that deals with disabling of REST support.
     * 
     * @param response
     *            the response
     * @throws IOException
     *             i/o exception
     */
    protected void showRestDisabledErrorMessage( HttpServletResponse response ) throws IOException
    {
        PrintWriter writer = new PrintWriter( response.getOutputStream() );
        writer.println( "<html><body><h2>Please enable REST support in WEB-INF/conf/axis2.xml "
            + "and WEB-INF/web.xml</h2></body></html>" );
        writer.flush();
        response.setStatus( HttpServletResponse.SC_ACCEPTED );
    }

    /**
     * Close the builders.
     * 
     * @param messageContext
     * @throws ServletException
     */
    private void closeStaxBuilder( MessageContext messageContext ) throws ServletException
    {
        if ( closeReader && messageContext != null )
        {
            try
            {
                SOAPEnvelope envelope = messageContext.getEnvelope();
                if ( envelope != null )
                {
                    StAXBuilder builder = (StAXBuilder) envelope.getBuilder();
                    if ( builder != null )
                    {
                        builder.close();
                    }
                }
            }
            catch ( Exception e )
            {
                LOG.debug( e.toString(), e );
            }
        }
    }

    /**
     * Processing for faults
     * 
     * @param msgContext
     * @param res
     * @param out
     * @param e
     */
    private void processAxisFault( MessageContext msgContext, HttpServletResponse res, OutputStream out,
                                   AxisFault e )
    {
        try
        {
            // If the fault is not going along the back channel we should be 202ing
            if ( AddressingHelper.isFaultRedirected( msgContext ) )
            {
                res.setStatus( HttpServletResponse.SC_ACCEPTED );
            }
            else
            {

                String status = (String) msgContext.getProperty( Constants.HTTP_RESPONSE_STATE );
                if ( status == null )
                {
                    res.setStatus( HttpServletResponse.SC_INTERNAL_SERVER_ERROR );
                }
                else
                {
                    res.setStatus( Integer.parseInt( status ) );
                }

                AxisBindingOperation axisBindingOperation =
                    (AxisBindingOperation) msgContext.getProperty( Constants.AXIS_BINDING_OPERATION );
                if ( axisBindingOperation != null )
                {
                    AxisBindingMessage fault =
                        axisBindingOperation.getFault( (String) msgContext.getProperty( Constants.FAULT_NAME ) );
                    if ( fault != null )
                    {
                        Integer code = (Integer) fault.getProperty( WSDL2Constants.ATTR_WHTTP_CODE );
                        if ( code != null )
                        {
                            res.setStatus( code.intValue() );
                        }
                    }
                }
            }
            handleFault( msgContext, out, e );
        }
        catch ( AxisFault e2 )
        {
            LOG.info( e2 );
        }
    }

    /**
     * 
     * @param msgContext
     *            the message context
     * @param out
     *            the response output stream
     * @param e
     *            the fault
     * @throws AxisFault
     *             AXIS fault
     */
    protected void handleFault( MessageContext msgContext, OutputStream out, AxisFault e ) throws AxisFault
    {
        msgContext.setProperty( MessageContext.TRANSPORT_OUT, out );

        MessageContext faultContext = MessageContextBuilder.createFaultMessageContext( msgContext, e );
        // SOAP 1.2 specification mentions that we should send HTTP code 400 in a fault if the
        // fault code Sender
        HttpServletResponse response =
            (HttpServletResponse) msgContext.getProperty( HTTPConstants.MC_HTTP_SERVLETRESPONSE );
        if ( response != null )
        {

            // TODO : Check for SOAP 1.2!
            SOAPFaultCode code = faultContext.getEnvelope().getBody().getFault().getCode();

            OMElement valueElement = null;
            if ( code != null )
            {
                valueElement =
                    code.getFirstChildWithName( new QName( SOAP12Constants.SOAP_ENVELOPE_NAMESPACE_URI,
                                                           SOAP12Constants.SOAP_FAULT_VALUE_LOCAL_NAME ) );
            }

            if ( valueElement != null )
            {
                if ( SOAP12Constants.FAULT_CODE_SENDER.equals( valueElement.getTextAsQName().getLocalPart() )
                    && !msgContext.isDoingREST() )
                {
                    response.setStatus( HttpServletResponse.SC_BAD_REQUEST );
                }
            }
        }

        AxisEngine.sendFault( faultContext );
    }

    /**
     * Main init method
     * 
     * @param config
     *            the servlet config
     * @throws ServletException
     *             servlet exception
     */
    public void init( ServletConfig config ) throws ServletException
    {

        // prevent this method from being called more than once per instance
        initCalled = true;
        super.init( config );
        try
        {
            this.servletConfig = config;
            ServletContext servletContext = servletConfig.getServletContext();
            this.configContext = (ConfigurationContext) servletContext.getAttribute( CONFIGURATION_CONTEXT );
            if ( configContext == null )
            {
                configContext = initConfigContext( config );
                config.getServletContext().setAttribute( CONFIGURATION_CONTEXT, configContext );
            }
            axisConfiguration = configContext.getAxisConfiguration();

            httpListener = getAxisServletListener( Constants.TRANSPORT_HTTP );
            httpsListener = getAxisServletListener( Constants.TRANSPORT_HTTPS );

            if ( httpListener == null && httpsListener == null )
            {
                LOG.warn( "No transportReceiver for " + AxisServletListener.class.getName()
                    + " found. An instance for HTTP will be configured automatically. "
                    + "Please update your axis2.xml file!" );
                httpListener = new AxisServletListener();
                TransportInDescription transportInDescription =
                    new TransportInDescription( Constants.TRANSPORT_HTTP );
                transportInDescription.setReceiver( httpListener );
                axisConfiguration.addTransportIn( transportInDescription );
            }
            else if ( httpListener != null && httpsListener != null && httpListener.getPort() == -1
                && httpsListener.getPort() == -1 )
            {
                LOG.warn( "If more than one transportReceiver for " + AxisServletListener.class.getName()
                    + " exists, then all instances "
                    + "must be configured with a port number. WSDL generation will be " + "unreliable." );
            }

            ListenerManager listenerManager = new ListenerManager();
            listenerManager.init( configContext );
            //
            // wsag4j fix
            //
            listenerManager.setShutdownHookRequired( false );
            listenerManager.start();
            ListenerManager.defaultConfigurationContext = configContext;
            agent = new ListingAgent( configContext );

            initParams();

        }
        catch ( Exception e )
        {
            throw new ServletException( e );
        }
    }

    private AxisServletListener getAxisServletListener( String name )
    {
        TransportInDescription desc = axisConfiguration.getTransportIn( name );
        if ( desc == null )
        {
            return null;
        }
        TransportListener receiver = desc.getReceiver();
        if ( receiver instanceof AxisServletListener )
        {
            return (AxisServletListener) receiver;
        }
        else
        {
            return null;
        }
    }

    /**
     * distroy the ConfigurationContext
     */
    public void destroy()
    {
        // stoping listner manager
        try
        {
            if ( configContext != null )
            {
                configContext.terminate();
            }
        }
        catch ( AxisFault axisFault )
        {
            LOG.info( axisFault.getMessage() );
        }
        try
        {
            super.destroy();
        }
        catch ( Exception e )
        {
            LOG.info( e.getMessage() );
        }

        axisConfiguration = null;
    }

    /**
     * Initializes the Axis2 parameters.
     */
    protected void initParams()
    {
        Parameter parameter;
        // do we need to completely disable REST support
        parameter = axisConfiguration.getParameter( Constants.Configuration.DISABLE_REST );
        if ( parameter != null )
        {
            disableREST = !JavaUtils.isFalseExplicitly( parameter.getValue() );
        }

        // Should we close the reader(s)
        parameter = axisConfiguration.getParameter( "axis2.close.reader" );
        if ( parameter != null )
        {
            closeReader = JavaUtils.isTrueExplicitly( parameter.getValue() );
        }

    }

    /**
     * Convenient method to re-initialize the ConfigurationContext
     * 
     * @throws ServletException
     *             the error
     */
    public void init() throws ServletException
    {
        if ( this.servletConfig != null && !initCalled )
        {
            init( this.servletConfig );
        }
    }

    /**
     * Initialize the Axis configuration context
     * 
     * @param config
     *            Servlet configuration
     * @return ConfigurationContext the configuration context
     * 
     * @throws ServletException
     *             a servlet exception
     */
    protected ConfigurationContext initConfigContext( ServletConfig config ) throws ServletException
    {
        try
        {
            ConfigurationContext configCtx =
                ConfigurationContextFactory.createConfigurationContext( new WarBasedAxisConfigurator( config ) );
            configCtx.setProperty( Constants.CONTAINER_MANAGED, Constants.VALUE_TRUE );
            return configCtx;
        }
        catch ( Exception e )
        {
            LOG.info( e );
            throw new ServletException( e );
        }
    }

    /**
     * Set the context root if it is not set already.
     * 
     * @param req
     *            the request
     */
    public void initContextRoot( HttpServletRequest req )
    {
        if ( contextRoot != null && contextRoot.trim().length() != 0 )
        {
            return;
        }
        String contextPath = req.getContextPath();
        // handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if ( contextPath != null && contextPath.length() == 0 )
        {
            contextPath = "/";
        }
        this.contextRoot = contextPath;

        configContext.setContextRoot( contextRoot );
    }

    /**
     * Preprocess the request. This will:
     * <ul>
     * <li>Set the context root if it is not set already (by calling
     * {@link #initContextRoot(HttpServletRequest)}).
     * <li>Remember the port number if port autodetection is enabled.
     * <li>Reject the request if no {@link AxisServletListener} has been registered for the protocol.
     * </ul>
     * 
     * @param req
     *            the request to preprocess
     */
    // This method should not be part of the public API. In particular we must not allow subclasses
    // to override this method because we don't make any guarantees as to when exactly this method
    // is called.
    private void preprocessRequest( HttpServletRequest req ) throws ServletException
    {
        initContextRoot( req );

        TransportInDescription transportInDescription =
            req.isSecure() ? this.axisConfiguration.getTransportIn( Constants.TRANSPORT_HTTPS )
                            : this.axisConfiguration.getTransportIn( Constants.TRANSPORT_HTTP );

        if ( transportInDescription == null )
        {
            throw new ServletException( req.getScheme() + " is forbidden" );
        }
        else
        {
            if ( transportInDescription.getReceiver() instanceof AxisServletListener )
            {
                AxisServletListener listner = (AxisServletListener) transportInDescription.getReceiver();
                // Autodetect the port number if necessary
                if ( listner.getPort() == -1 )
                {
                    listner.setPort( req.getServerPort() );
                }
            }
        }

    }

    /**
     * Get all transport headers.
     * 
     * @param req
     *            the request
     * @return Map the transport headers
     */
    protected Map<?, ?> getTransportHeaders( HttpServletRequest req )
    {
        return new TransportHeaders( req );
    }

    /**
     * @param request
     *            the request
     * @param response
     *            the response
     * @param invocationType
     *            If invocationType=true; then this will be used in SOAP message invocation. If
     *            invocationType=false; then this will be used in REST message invocation.
     * @return MessageContext the message context
     * 
     * @throws IOException
     *             IO error
     */
    protected MessageContext createMessageContext( HttpServletRequest request, HttpServletResponse response,
                                                   boolean invocationType ) throws IOException
    {
        MessageContext msgContext = configContext.createMessageContext();
        String requestURI = request.getRequestURI();

        String trsPrefix = request.getRequestURL().toString();
        int sepindex = trsPrefix.indexOf( ':' );
        if ( sepindex > -1 )
        {
            trsPrefix = trsPrefix.substring( 0, sepindex );
            msgContext.setIncomingTransportName( trsPrefix );
        }
        else
        {
            msgContext.setIncomingTransportName( Constants.TRANSPORT_HTTP );
            trsPrefix = Constants.TRANSPORT_HTTP;
        }
        TransportInDescription transportIn =
            axisConfiguration.getTransportIn( msgContext.getIncomingTransportName() );
        // set the default output description. This will be http

        TransportOutDescription transportOut = axisConfiguration.getTransportOut( trsPrefix );
        if ( transportOut == null )
        {
            // if the req coming via https but we do not have a https sender
            transportOut = axisConfiguration.getTransportOut( Constants.TRANSPORT_HTTP );
        }

        msgContext.setTransportIn( transportIn );
        msgContext.setTransportOut( transportOut );
        msgContext.setServerSide( true );

        if ( !invocationType )
        {
            String query = request.getQueryString();
            if ( query != null )
            {
                requestURI = requestURI + "?" + query;
            }
        }

        msgContext.setTo( new EndpointReference( requestURI ) );
        msgContext.setFrom( new EndpointReference( request.getRemoteAddr() ) );
        msgContext.setProperty( MessageContext.REMOTE_ADDR, request.getRemoteAddr() );
        msgContext.setProperty( Constants.OUT_TRANSPORT_INFO, new ServletBasedOutTransportInfo( response ) );
        // set the transport Headers
        msgContext.setProperty( MessageContext.TRANSPORT_HEADERS, getTransportHeaders( request ) );
        msgContext.setProperty( HTTPConstants.MC_HTTP_SERVLETREQUEST, request );
        msgContext.setProperty( HTTPConstants.MC_HTTP_SERVLETRESPONSE, response );
        try
        {
            ServletContext context = getServletContext();
            if ( context != null )
            {
                msgContext.setProperty( HTTPConstants.MC_HTTP_SERVLETCONTEXT, context );
            }
        }
        catch ( Exception e )
        {
            LOG.debug( e.getMessage(), e );
        }

        // setting the RequestResponseTransport object
        msgContext.setProperty( RequestResponseTransport.TRANSPORT_CONTROL,
                                new ServletRequestResponseTransport( response ) );

        return msgContext;
    }

    /**
     * This method assumes, that the created MessageContext will be used in only SOAP invocation.
     * 
     * @param req
     *            the request
     * @param resp
     *            the response
     * @return MessageContext the message context
     * 
     * @throws IOException
     *             an IO error
     */

    protected MessageContext createMessageContext( HttpServletRequest req, HttpServletResponse resp )
        throws IOException
    {
        return createMessageContext( req, resp, true );
    }

    /**
     * 
     * 
     */
    protected class ServletRequestResponseTransport implements RequestResponseTransport
    {
        @SuppressWarnings( "unused" )
        private HttpServletResponse response;

        private boolean responseWritten = false;

        private CountDownLatch responseReadySignal = new CountDownLatch( 1 );

        // The initial status must be WAITING, as the main servlet will do some other
        // work after setting this RequestResponseTransport up, and we don't want to miss
        // signals that come in before this thread gets to the awaitResponse call.
        private RequestResponseTransportStatus status = RequestResponseTransportStatus.WAITING;

        AxisFault faultToBeThrownOut = null;

        ServletRequestResponseTransport( HttpServletResponse response )
        {
            this.response = response;
        }

        /**
         * {@inheritDoc}
         */
        public void acknowledgeMessage( MessageContext msgContext ) throws AxisFault
        {
            status = RequestResponseTransportStatus.ACKED;
            responseReadySignal.countDown();
        }

        /**
         * {@inheritDoc}
         */
        public void awaitResponse() throws InterruptedException, AxisFault
        {
            LOG.debug( "Blocking servlet thread -- awaiting response" );
            responseReadySignal.await();

            if ( faultToBeThrownOut != null )
            {
                throw faultToBeThrownOut;
            }
        }

        /**
         * 
         */
        public void signalResponseReady()
        {
            LOG.debug( "Signalling response available" );
            status = RequestResponseTransportStatus.SIGNALLED;
            responseReadySignal.countDown();
        }

        /**
         * @return the status
         */
        public RequestResponseTransportStatus getStatus()
        {
            return status;
        }

        /**
         * @param fault
         *            the Axis fault
         */
        public void signalFaultReady( AxisFault fault )
        {
            faultToBeThrownOut = fault;
            signalResponseReady();
        }

        /**
         * @return response written
         */
        public boolean isResponseWritten()
        {
            return responseWritten;
        }

        /**
         * @param responseWritten
         *            the response written to set
         */
        public void setResponseWritten( boolean responseWritten )
        {
            this.responseWritten = responseWritten;
        }

    }

    private void setResponseState( MessageContext messageContext, HttpServletResponse response )
    {
        String state = (String) messageContext.getProperty( Constants.HTTP_RESPONSE_STATE );
        if ( state != null )
        {
            int stateInt = Integer.parseInt( state );
            if ( stateInt == HttpServletResponse.SC_UNAUTHORIZED )
            { // Unauthorized
                String realm = (String) messageContext.getProperty( Constants.HTTP_BASIC_AUTH_REALM );
                response.addHeader( "WWW-Authenticate", "basic realm=\"" + realm + "\"" );
            }
        }
    }

    /**
     * Ues in processing REST related Requests. This is the helper Class use in processing of doGet, doPut ,
     * doDelete and doPost.
     */
    protected class RestRequestProcessor
    { 
        /**
         * Message Context 
         */
        protected MessageContext messageContext;

        private HttpServletRequest request;

        private HttpServletResponse response;

        /**
         * 
         * @param httpMethodString
         *            the HTTP method
         * @param request
         *            the request
         * @param response
         *            the response
         * 
         * @throws IOException
         *             IOException
         */
        public RestRequestProcessor( String httpMethodString, HttpServletRequest request,
                                     HttpServletResponse response )
            throws IOException
        {
            this.request = request;
            this.response = response;
            messageContext = createMessageContext( this.request, this.response, false );
            messageContext.setProperty( org.apache.axis2.transport.http.HTTPConstants.HTTP_METHOD,
                                        httpMethodString );
        }

        /**
         * 
         * @throws IOException
         *             IOException
         * @throws ServletException
         *             ServletException
         */
        public void processXMLRequest() throws IOException, ServletException
        {
            try
            {
                RESTUtil.processXMLRequest( messageContext, request.getInputStream(),
                                            response.getOutputStream(), request.getContentType() );
                this.checkResponseWritten();
            }
            catch ( AxisFault axisFault )
            {
                processFault( axisFault );
            }
            closeStaxBuilder( messageContext );
        }

        /**
         * 
         * @throws IOException
         *             IOException
         * @throws ServletException
         *             ServletException
         */
        public void processURLRequest() throws IOException, ServletException
        {
            try
            {
                RESTUtil.processURLRequest( messageContext, response.getOutputStream(),
                                            request.getContentType() );
                this.checkResponseWritten();
            }
            catch ( AxisFault e )
            {
                setResponseState( messageContext, response );
                processFault( e );
            }
            closeStaxBuilder( messageContext );

        }

        private void checkResponseWritten()
        {
            if ( !TransportUtils.isResponseWritten( messageContext ) )
            {
                response.setStatus( HttpServletResponse.SC_ACCEPTED );
            }
        }

        private void processFault( AxisFault e ) throws ServletException, IOException
        {
            LOG.debug( e );
            if ( messageContext != null )
            {
                processAxisFault( messageContext, response, response.getOutputStream(), e );
            }
            else
            {
                throw new ServletException( e );
            }

        }

    }

    // private static final Logger log = Logger.getLogger(Bootstrap.class);
    //
    // private static final long serialVersionUID = 1L;
    //
    // private BootstrapIsolationLayer isolationLayer;
    //
    // /* (non-Javadoc)
    // * @see org.apache.axis2.transport.http.AxisServlet#init()
    // */
    // public void init(ServletConfig config) throws ServletException {
    // super.init(config);
    // // try {
    // // //
    // // // get the real path for bootstraping configuration files, etc.
    // // //
    // // ServletContext context = config.getServletContext();
    // // String path = context.getRealPath("/");
    // // String defaultGatewayURI = getDefaultGatewayURI(context);
    // // String datapath = path + "/WEB-INF/wsag4j-data";
    // //
    // // if (!System.getProperties().containsKey(EmfRegistry.WSAG4J_DATAPATH)) {
    // // System.getProperties().setProperty(EmfRegistry.WSAG4J_DATAPATH, datapath);
    // // }
    // //
    // // //
    // // // create and initialize the isolation layer
    // // //
    // // isolationLayer = new BootstrapIsolationLayer(path, defaultGatewayURI);
    // // isolationLayer.initialize();
    // //
    // // }
    // // catch (Exception e) {
    // // String message = "WSAG4J initialization process failed. Error: " + e.getMessage();
    // // log.error(message);
    // //
    // // throw new ServletException(message, e);
    // // }
    // }
    //
    // public void destroy() {
    // // log.info("shutdown WSAG4J service");
    // //
    // // try {
    // // isolationLayer.shutdown();
    // // isolationLayer = null;
    // // }
    // // catch(Exception e) {
    // // String message = "WSAG4J shutdown process failed. Error: " + e.getMessage();
    // // log.error(message);
    // // }
    // //
    // // BootstrapRegistry.finalizeBootstrapRegistry();
    // //
    // // DatabaseManager.getTimer().shutDown();
    // //
    // // Introspector.flushCaches();
    // // for (Enumeration<Driver> e = DriverManager.getDrivers(); e.hasMoreElements();) {
    // // Driver driver = e.nextElement();
    // // if (driver.getClass().getClassLoader() == getClass().getClassLoader()) {
    // // try {
    // // DriverManager.deregisterDriver(driver);
    // // }
    // // catch (Exception ex) {
    // // log.warn("failed to deregister driver.", ex);
    // // }
    // // }
    // // }
    // //
    // // delegate destroy call
    // super.destroy();
    //
    // axisConfiguration = null;
    // //
    // // log.info("shutdown of WSAG4J service completed");
    // }
    //
    // // protected String getDefaultGatewayURI(ServletContext context) throws Exception {
    // // String contextPath = context.getResource("/").getPath();
    // // contextPath = contextPath.substring(0, contextPath.lastIndexOf("/"));
    // // contextPath = contextPath.substring(contextPath.lastIndexOf("/") + 1);
    // //
    // // String host = InetAddress.getLocalHost().getHostAddress();
    // // ConfigurationContext axisContext =
    // (ConfigurationContext)context.getAttribute(CONFIGURATION_CONTEXT);
    // //
    // // // Try to resolve the running port via the Axis2 configuration.
    // // // We use 8080 as default port.
    // // String port = (String)axisContext.getProperty(ListingAgent.RUNNING_PORT);
    // // port = (port == null) ? "8080" : port;
    // //
    // // String generatedGatewayURL = "http://" + host + ":" + port + "/" + contextPath;
    // //
    // // return generatedGatewayURL;
    // // }
    //
    // /* (non-Javadoc)
    // * @see org.apache.axis2.transport.http.AxisServlet#doGet(javax.servlet.http.HttpServletRequest,
    // javax.servlet.http.HttpServletResponse)
    // */
    // protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException,
    // IOException {
    // //response.getOutputStream().println("<!-- Invokation of Bootstrap Service -->");
    // super.doGet(request, response);
    // }
    // CHECKSTYLE:ON
}
