/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.bootstrap;

import java.io.File;
import java.io.InputStream;
import java.net.URL;

import org.apache.muse.core.AbstractEnvironment;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.MessageHeaders;

/**
 * The bootstrap environment implements a {@link AbstractEnvironment} in order to start up the WSRF layer
 * before the initial invocation of the web service interface.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class BootstrapEnvironment extends AbstractEnvironment
{

    private String realPath = null;

    /**
     * Creates a {@link BootstrapEnvironment} with a given application path (path of the expanded web
     * application) and the external URI of the application.
     * 
     * @param realPath
     *            the real path on the file system used for this environment
     * 
     * @param defaultDeploymentUri
     *            the default WSRF deployment URI
     */
    public BootstrapEnvironment( String realPath, String defaultDeploymentUri )
    {
        this.realPath = realPath;
        setDefaultURI( defaultDeploymentUri );
    }

    /**
     * @return the deployment EPR
     */
    public EndpointReference getDeploymentEPR()
    {
        return getDefaultEPR();
    }

    /**
     * @return the real path on the file system used for this environment
     */
    public File getRealDirectory()
    {
        return new File( realPath, "WEB-INF/classes" );
    }

    /**
     * registers a new addressing context
     * 
     * @param context
     *            the addressing context
     */
    public synchronized void addAddressingContext( MessageHeaders context )
    {
        BootstrapRegistry.registerAddressingContext( context );
    }

    /**
     * @return the registered addressing context
     */
    public synchronized MessageHeaders getAddressingContext()
    {
        return BootstrapRegistry.getAddressingContext();
    }

    /**
     * looks up a data resource at the given path
     * 
     * @param path
     *            the name of the data resource
     * 
     * @return the data resource URL
     */
    public URL getDataResource( String path )
    {
        return super.getDataResource( resolveName( path ) );
    }

    /**
     * Returns the data resource at the given path as an {@link InputStream}.
     * 
     * @param path
     *            the name of the data resource
     * 
     * @return the data resource input stream
     */
    public InputStream getDataResourceStream( String path )
    {
        String resource = resolveName( path );
        try
        {
            return super.getDataResourceStream( resource );
        }
        catch ( RuntimeException e )
        {
            throw new RuntimeException( "Could not reolve data resource: " + resource, e );
        }
    }

    /**
     * Remove leading "/" if name is absolute
     */
    private String resolveName( String name )
    {
        if ( name == null )
        {
            return name;
        }

        if ( name.startsWith( "/" ) )
        {
            name = name.substring( 1 );
        }
        return name;
    }

}
