/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.bootstrap;

import javax.security.auth.login.LoginContext;

import org.apache.log4j.Logger;
import org.apache.muse.core.Environment;
import org.apache.muse.core.descriptor.DeploymentDescriptor;
import org.apache.muse.core.platform.AbstractIsolationLayer;
import org.ogf.graap.wsag.security.core.keystore.KeystoreLoginContext;
import org.ogf.graap.wsag.server.engine.WsagEngine;

/**
 * The bootstrap isolation layer implements the {@link AbstractIsolationLayer} in order to start up the WSRF
 * layer before the initial invocation of the web service interface.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class BootstrapIsolationLayer extends AbstractIsolationLayer
{

    private static final Logger LOG = Logger.getLogger( BootstrapIsolationLayer.class );

    private String realPath = null;

    private String deplomentUri = null;

    private boolean testMode = false;

    private boolean overrideHasBeenInitializedValue = false;

    private boolean overrideHasBeenShutdownValue = false;

    private boolean overrideHasFailedToInitializeValue = false;

    /**
     * Creates a {@link BootstrapIsolationLayer} with a given application path (path of the expanded web
     * application) and the external URI of the application.
     * 
     * @param realPath
     *            the real path at the file system
     * 
     * @param defaultDeploymentUri
     *            the default deployment URL
     */
    public BootstrapIsolationLayer( String realPath, String defaultDeploymentUri )
    {
        this.realPath = realPath;
        deplomentUri = defaultDeploymentUri;

        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( "create bootstrap isolation layer [real path: " + realPath + ", deployment uri: "
                + defaultDeploymentUri + "]" );
        }
    }

    /**
     * Creates a new environment instance.
     * 
     * @return the new environment instance
     */
    protected Environment createEnvironment()
    {
        return new BootstrapEnvironment( realPath, WsagEngine.getGatewayURL() + "/services/WSAG4JService" );
    }

    @Override
    protected DeploymentDescriptor createDeploymentDescriptor()
    {
        return new WSAG4JDeploymentDescriptor();
    }

    /**
     * Initializes the environment.
     */
    public void initialize()
    {

        LOG.info( "BootstrapIsolationLayer -> initialize" );

        synchronized ( BootstrapIsolationLayer.class )
        {
            if ( !hasBeenInitialized() )
            {
                try
                {
                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "Initialize WsagEngine..." );
                    }

                    //
                    // initialize the WSAG4J server configuration
                    //
                    WsagEngine.initializeEngine( deplomentUri );

                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "Build LoginContext..." );
                    }

                    //
                    // load the server login context
                    //
                    LoginContext context = new KeystoreLoginContext( WsagEngine.getWSRFConfiguration() );
                    context.login();

                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "Set LoginContext..." );
                    }

                    //
                    // set the login context for the engine
                    //
                    WsagEngine.setLoginContext( context );

                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "Build LoginContext..." );
                        LOG.debug( "Delegate initialization to AbstractIsolationLayer..." );
                    }

                    //
                    // call hook for subclasses
                    //
                    beforeParentInitialize();

                    //
                    // initialize the isolation layer
                    //
                    super.initialize();

                    // set initialize override value
                    setOverrideHasBeenInitializedValue( true );
                }
                catch ( Exception ex )
                {
                    throw new RuntimeException(
                                                "The initialization process of the WSAG4J engine failed. Reason: "
                                                    + ex.getMessage(), ex );
                }
            }
        }
    }

    /**
     * Shuts down the isolation layer.
     */
    public void shutdown()
    {

        LOG.info( "BootstrapIsolationLayer -> shutdown" );

        synchronized ( BootstrapIsolationLayer.class )
        {
            if ( !hasBeenShutdown() )
            {
                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( "Shutting down WSAG4JResourceRouter..." );
                }

                try
                {
                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "Delegate shutdown call..." );
                    }

                    // shutdown the isolation layer
                    super.shutdown();

                    // set shutdown override value
                    setOverrideHasBeenShutdownValue( true );

                    // shutdown the WSAG4J instance (the server)
                    WsagEngine.shutdownEngine();

                }
                catch ( Exception ex )
                {
                    throw new RuntimeException( "The shutdown process of the WSAG4J engine failed. Reason: "
                        + ex.getMessage(), ex );
                }
            }
        }
    }

    /**
     * Hook for sub classes that is called before initialization.
     */
    protected void beforeParentInitialize()
    {
    }

    /**
     * Toggles test mode on/off.
     * 
     * @param testMode
     *            sets the test mode
     */
    public void setTestMode( boolean testMode )
    {
        this.testMode = testMode;
    }

    @Override
    public boolean hasBeenInitialized()
    {
        if ( testMode )
        {
            return overrideHasBeenInitializedValue;
        }
        else
        {
            return super.hasBeenInitialized();
        }
    }

    @Override
    public boolean hasBeenShutdown()
    {
        if ( testMode )
        {
            return overrideHasBeenShutdownValue;
        }
        else
        {
            return super.hasBeenShutdown();
        }
    }

    @Override
    public boolean hasFailedToInitialize()
    {
        if ( testMode )
        {
            return overrideHasFailedToInitializeValue;
        }
        else
        {
            return super.hasFailedToInitialize();
        }
    }

    /**
     * 
     * @return true if has been initialized
     */
    public boolean isOverrideHasBeenInitializedValue()
    {
        return overrideHasBeenInitializedValue;
    }

    /**
     * 
     * @param overrideHasBeenInitializedValue
     *            sets the overrideHasBeenInitializedValue
     */
    public void setOverrideHasBeenInitializedValue( boolean overrideHasBeenInitializedValue )
    {
        this.overrideHasBeenInitializedValue = overrideHasBeenInitializedValue;
    }

    /**
     * 
     * @return true if has been shutdown
     */
    public boolean isOverrideHasBeenShutdownValue()
    {
        return overrideHasBeenShutdownValue;
    }

    /**
     * @param overrideHasBeenShutdownValue
     *            sets the overrideHasBeenShutdownValue
     */
    public void setOverrideHasBeenShutdownValue( boolean overrideHasBeenShutdownValue )
    {
        this.overrideHasBeenShutdownValue = overrideHasBeenShutdownValue;
    }

    /**
     * 
     * @return true if has failed to initialize
     */
    public boolean isOverrideHasFailedToInitializeValue()
    {
        return overrideHasFailedToInitializeValue;
    }

    /**
     * @param overrideHasFailedToInitializeValue
     *            sets the overrideHasFailedToInitializeValue
     */
    public void setOverrideHasFailedToInitializeValue( boolean overrideHasFailedToInitializeValue )
    {
        this.overrideHasFailedToInitializeValue = overrideHasFailedToInitializeValue;
    }
}
