/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.faults;

import java.util.GregorianCalendar;

import javax.xml.namespace.QName;

import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.util.xml.XsdUtils;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.resource.WsrfConstants;
import org.apache.muse.ws.resource.basefaults.BaseFault;
import org.apache.muse.ws.resource.basefaults.WsbfConstants;
import org.ogf.schemas.graap.wsAgreement.ContinuingFaultDocument;
import org.ogf.schemas.graap.wsAgreement.ContinuingFaultType;
import org.w3c.dom.Element;

/**
 * AbstractAgreementFault
 * 
 * @author Oliver Waeldrich
 * 
 */
public abstract class AbstractAgreementFault extends BaseFault
{

    private static final long serialVersionUID = 1L;

    /**
     * 
     * @param element
     *            the fault element
     */
    public AbstractAgreementFault( Element element )
    {
        super( element );
    }

    /**
     * 
     * @param name
     *            fault name
     * 
     * @param message
     *            error message
     */
    public AbstractAgreementFault( QName name, String message )
    {
        super( name, message );
    }

    /**
     * 
     * @param name
     *            fault name
     * @param message
     *            error message
     * @param throwable
     *            cause
     */
    public AbstractAgreementFault( QName name, String message, Throwable throwable )
    {
        super( name, message, throwable );
    }

    /**
     * 
     * @param name
     *            fault name
     * @param throwable
     *            cause
     */
    public AbstractAgreementFault( QName name, Throwable throwable )
    {
        super( name, throwable );
    }

    /**
     * Creates the detail section of the fault with timestamp and description.
     */
    protected void createDetail()
    {
        Element baseFault = XmlUtils.createElement( getName() );

        Element timestamp =
            XmlUtils.createElement( new QName( WsrfConstants.NAMESPACE_URI, "Timestamp", WsrfConstants.PREFIX ),
                                    XsdUtils.getLocalTimeString( getTimestamp() ) );

        Element description =
            XmlUtils.createElement( new QName( WsrfConstants.NAMESPACE_URI, "Description",
                                               WsrfConstants.PREFIX ), getMessage() );

        baseFault.appendChild( timestamp );
        baseFault.appendChild( description );

        setDetail( baseFault );
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.muse.ws.resource.basefaults.BaseFault#getDetail()
     */
    @Override
    public Element getDetail()
    {
        Element baseFault = XmlUtils.createElement( getName() );
        XmlUtils.setElement( baseFault, WsbfConstants.TIMESTAMP_QNAME, getTimestamp() );

        EndpointReference originEPR = getOriginReference();

        if ( originEPR != null )
        {
            Element originXML = originEPR.toXML();
            baseFault.appendChild( originXML );
        }

        Element errorCode = getErrorCode();

        if ( errorCode != null )
        {
            Element errorXML = XmlUtils.createElement( WsbfConstants.ERROR_CODE_QNAME );
            errorXML.setAttribute( WsbfConstants.ERROR_CODE_DIALECT, getErrorCodeDialect() );
            XmlUtils.setElement( errorXML, WsbfConstants.ERROR_CODE_QNAME, errorCode, false );
            baseFault.appendChild( errorXML );
        }

        try
        {
            //
            // root cause message is included into the SOAP error,
            // we try to go one level deeper.
            //
            Throwable cause = getCause().getCause();
            if ( cause != null )
            {
                ContinuingFaultType fault =
                    ContinuingFaultDocument.Factory.newInstance().addNewContinuingFault();
                fault.addNewDescription();
                fault.getDescriptionArray( 0 ).setLang( "en" );
                fault.getDescriptionArray( 0 ).setStringValue( cause.getMessage() );
                GregorianCalendar cal = new GregorianCalendar();
                cal.setTime( getTimestamp() );
                fault.setTimestamp( cal );
                Element causeXML = XmlUtils.createElement( WsbfConstants.FAULT_CAUSE_QNAME );
                XmlUtils.setElement( causeXML, ContinuingFaultDocument.type.getDocumentElementName(),
                                     fault.getDomNode(), true );
                XmlUtils.setElement( baseFault, WsbfConstants.FAULT_CAUSE_QNAME, causeXML, true );
            }
        }
        catch ( Exception e )
        {
            // do nothing
        }

        return baseFault;
    }
}
