/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.handler;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.Vector;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.OMNodeEx;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.om.util.StAXUtils;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.util.StreamWrapper;
import org.apache.log4j.Logger;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.WsaConstants;
import org.apache.muse.ws.addressing.soap.SoapConstants;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.xmlbeans.XmlObject;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Handles incoming SOAP messages and invokes WSRF stack. In case of an error, the exception is handled for
 * correct serialization.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AxisHandler
{

    MuseMessageHandler delegate = new MuseMessageHandler();

    private static final Logger LOG = Logger.getLogger( AxisHandler.class );

    /**
     * Invokes WSRF stack and handles errors from the WSRF stack. The WSRF stack returns SOAP errors already
     * as XML, we therefore check the QName of the response and in case of error the proper Axis fault is
     * created.
     * 
     * @param request
     *            the Axis2 request
     * 
     * @return the Axis2 response message
     * 
     * @throws AxisFault
     *             the converted axis fault in case a SOAP error was created by the WSRF environment.
     */
    @SuppressWarnings( { "unchecked" } )
    public final OMElement handleRequest( OMElement request ) throws AxisFault
    {

        LOG.trace( "entering WSAG4J AxisHandler" );

        try
        {

            OMElement response = delegate.handleRequest( request );

            //
            // if the operation was a fault, there will always be
            // a SOAP 1.2 Fault element in the response body
            //
            // OMElement fault = envelope.getBody().getFirstChildWithName(SoapConstants.FAULT_QNAME);
            if ( ( response.getQName().equals( SoapConstants.FAULT_QNAME ) ) )
            {
                //
                // the response contains an error
                //
                MessageContext msgContext = MessageContext.getCurrentMessageContext();

                StringWriter writer = new StringWriter();
                try
                {
                    XMLStreamWriter writer2 = StAXUtils.createXMLStreamWriter( writer );
                    try
                    {
                        response.serialize( writer2 );
                        writer2.flush();
                    }
                    finally
                    {
                        writer2.close();
                    }
                }
                catch ( XMLStreamException e )
                {
                    throw new AxisFault( "Can not serialize OM Element in WSAG4J error handling", e );
                }

                InputStream in = new ByteArrayInputStream( writer.toString().getBytes() );

                msgContext.setWSAAction( WsaConstants.FAULT_URI );

                msgContext.setProperty( org.apache.axis2.Constants.FAULT_NAME, "BaseFault" );

                AxisFault f = null;
                try
                {
                    XmlObject parsedFault = XmlObject.Factory.parse( in );

                    Element node = null;

                    if ( parsedFault.getDomNode() instanceof Element )
                    {
                        node = (Element) parsedFault.getDomNode();
                    }
                    else if ( parsedFault.getDomNode() instanceof Document )
                    {
                        node = XmlUtils.getFirstElement( parsedFault.getDomNode() );
                    }

                    if ( node == null )
                    {
                        String message = "Parsed fault must no be null: " + parsedFault.xmlText();
                        throw new Exception( message );
                    }

                    SoapFault fault = new SoapFault( (Element) node );

                    String message = fault.getMessage();
                    Throwable cause = fault.getCause();

                    if ( cause == null )
                    {
                        f = new AxisFault( message );
                    }
                    else
                    {
                        f = new AxisFault( message, cause );
                    }

                    if ( fault.getCode() != null )
                    {
                        f.setFaultCode( fault.getCode() );
                    }

                    if ( fault.getSubCode() != null )
                    {
                        f.setFaultSubCodes( new Vector<QName>() );
                        f.getFaultSubCodes().add( fault.getSubCode() );
                    }

                    if ( fault.getDetail() != null )
                    {
                        XmlObject detail = XmlObject.Factory.parse( fault.getDetail() );

                        StAXOMBuilder builder =
                            new StAXOMBuilder( OMAbstractFactory.getOMFactory(),
                                               new StreamWrapper( detail.newXMLStreamReader() ) );
                        OMElement documentElement = builder.getDocumentElement();

                        ( (OMNodeEx) documentElement ).setParent( null );

                        f.setDetail( documentElement );
                    }
                }
                catch ( Exception ex )
                {
                    String message = "Error in WSAG4J Fault handling.";
                    LOG.error( message );
                    LOG.error( ex );
                    throw new AxisFault( message, ex );
                }

                throw f;
            }

            return response;
        }
        finally
        {
            LOG.trace( "leaving WSAG4J AxisHandler" );
        }
    }
}
