/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.faults.ResourceUnavailableFault;
import org.apache.muse.ws.resource.faults.ResourceUnknownFault;
import org.apache.ws.security.WSSecurityException;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.exceptions.WSAgreementException;
import org.ogf.graap.wsag.server.api.WsagSession;
import org.ogf.graap.wsag.server.engine.WsagEngine;
import org.ogf.graap.wsag.wsrf.WSAG4JCapability;
import org.ogf.schemas.graap.wsAgreement.AgreementContextType;
import org.ogf.schemas.graap.wsAgreement.AgreementIdDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementRoleType;
import org.ogf.schemas.graap.wsAgreement.AgreementStateDocument;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateDocument;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateType;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateDocument;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateType;
import org.ogf.schemas.graap.wsAgreement.TemplateDocument;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputDocument;
import org.ogf.schemas.graap.wsAgreement.TerminateOutputType;
import org.ogf.schemas.graap.wsAgreement.TerminateResponseDocument;
import org.w3c.dom.Element;

/**
 * Default implementation of the WS-Agreement port type.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AgreementCapability extends WSAG4JCapability
{

    @SuppressWarnings( "unused" )
    private static Logger log = Logger.getLogger( AgreementCapability.class );

    private Agreement getAgreement() throws WSSecurityException
    {
        return ( (AgreementWsResource) getResource() ).getAgreement();
    }

    private WsagSession getSession()
    {
        return ( (AgreementWsResource) getResource() ).getSession();
    }

    /**
     * @return the resource property QNames
     */
    public QName[] getPropertyNames()
    {
        return WsagConstants.WSAG_AGREEMENT_PROPERTIES;
    }

    /**
     * Terminates an agreement.
     * 
     * @param input
     *            the input document, which may contain an optional termination reason
     * 
     * @return the response document, this is an empty document
     * 
     * @throws SoapFault
     *             indicates a fault terminating the agreement
     */
    public TerminateOutputType terminate( TerminateInputDocument input ) throws SoapFault
    {
        try
        {
            WsagEngine.getWsagMessageContext().setSession( getSession() );

            TerminateResponseDocument response = TerminateResponseDocument.Factory.newInstance();
            getAgreement().terminate( input.getTerminateInput() );
            response.addNewTerminateResponse();

            return response.getTerminateResponse();

        }
        catch ( Exception e )
        {
            throw new SoapFault( e );
        }
    }

    /**
     * 
     * @return the agreement name
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public String getName() throws ResourceUnknownFault, ResourceUnavailableFault
    {
        WsagEngine.getWsagMessageContext().setSession( getSession() );

        try
        {
            return getAgreement().getName();
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * 
     * @return the agreement id
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public Element getAgreementId() throws ResourceUnknownFault, ResourceUnavailableFault
    {
        try
        {
            WsagEngine.getWsagMessageContext().setSession( getSession() );

            String id = getAgreement().getAgreementId();
            AgreementIdDocument doc = AgreementIdDocument.Factory.newInstance();
            doc.setAgreementId( id );

            return toElement( doc );

        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * @return the agreement context
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public Element getContext() throws ResourceUnknownFault, ResourceUnavailableFault
    {
        try
        {
            WsagEngine.getWsagMessageContext().setSession( getSession() );

            AgreementContextType context = getAgreement().getContext();
            if ( context == null )
            {
                context = TemplateDocument.Factory.newInstance().addNewTemplate().addNewContext();

                Calendar expire = new GregorianCalendar();
                expire.add( Calendar.DAY_OF_YEAR, 1 );

                context.setExpirationTime( expire );
                context.setServiceProvider( AgreementRoleType.AGREEMENT_RESPONDER );
            }

            return toElement( context );
        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * @return the agreement terms
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public Element getTerms() throws ResourceUnknownFault, ResourceUnavailableFault
    {
        try
        {
            WsagEngine.getWsagMessageContext().setSession( getSession() );

            TermTreeType terms = getAgreement().getTerms();

            if ( terms == null )
            {
                terms = TemplateDocument.Factory.newInstance().addNewTemplate().addNewTerms();
                terms.addNewAll();
            }

            return toElement( terms );
        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * @return the agreement state
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public Element getAgreementState() throws ResourceUnknownFault, ResourceUnavailableFault
    {
        try
        {
            WsagEngine.getWsagMessageContext().setSession( getSession() );

            AgreementStateDocument state = AgreementStateDocument.Factory.newInstance();
            state.addNewAgreementState();

            if ( getAgreement().getState() != null )
            {
                state.getAgreementState().set( getAgreement().getState() );
            }

            return toElement( state );
        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * @return the agreement guarantee term states
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public Element[] getGuaranteeTermState() throws ResourceUnknownFault, ResourceUnavailableFault
    {
        try
        {
            WsagEngine.getWsagMessageContext().setSession( getSession() );

            GuaranteeTermStateType[] currentStates = getAgreement().getGuaranteeTermStates();

            if ( currentStates == null )
            {
                currentStates = new GuaranteeTermStateType[0];
            }

            GuaranteeTermStateDocument[] states = new GuaranteeTermStateDocument[currentStates.length];
            for ( int i = 0; i < states.length; i++ )
            {
                states[i] = GuaranteeTermStateDocument.Factory.newInstance();
                states[i].setGuaranteeTermState( currentStates[i] );
            }

            return toElementArray( states );
        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * @return the agreement service term states
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public Element[] getServiceTermState() throws ResourceUnknownFault, ResourceUnavailableFault
    {
        try
        {
            WsagEngine.getWsagMessageContext().setSession( getSession() );

            ServiceTermStateType[] currentStates = getAgreement().getServiceTermStates();
            if ( currentStates == null )
            {
                currentStates = new ServiceTermStateType[0];
            }

            ServiceTermStateDocument[] states = new ServiceTermStateDocument[currentStates.length];
            for ( int i = 0; i < states.length; i++ )
            {
                states[i] = ServiceTermStateDocument.Factory.newInstance();
                states[i].setServiceTermState( currentStates[i] );
            }

            return toElementArray( states );

        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

}
