/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import javax.persistence.EntityManager;
import javax.persistence.RollbackException;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.core.Resource;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.resource.faults.ResourceUnavailableFault;
import org.apache.muse.ws.resource.faults.ResourceUnknownFault;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.exceptions.WSAgreementException;
import org.ogf.graap.wsag.security.core.SecurityConstants;
import org.ogf.graap.wsag.server.api.WsagSession;
import org.ogf.graap.wsag.server.api.WsagSessionManager;
import org.ogf.graap.wsag.server.engine.WsagEngine;
import org.ogf.graap.wsag.server.persistence.EmfRegistry;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.graap.wsag.wsrf.AgreementAcceptanceClient;
import org.ogf.graap.wsag.wsrf.WSAG4JCapability;
import org.ogf.graap.wsag.wsrf.XmlUtils;
import org.ogf.graap.wsag.wsrf.faults.AgreementFactoryFault;
import org.ogf.graap.wsag.wsrf.persistence.AgreementEprContainer;
import org.ogf.graap.wsag4j.types.engine.WSAG4JSessionDocument;
import org.ogf.graap.wsag4j.types.engine.WSAG4JSessionType;
import org.ogf.schemas.graap.wsAgreement.AgreementContextType;
import org.ogf.schemas.graap.wsAgreement.AgreementFactoryPropertiesDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.AgreementType;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementInputDocument;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementInputType;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementOutputType;
import org.ogf.schemas.graap.wsAgreement.CreateAgreementResponseDocument;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementInputDocument;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementInputType;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementOutputType;
import org.ogf.schemas.graap.wsAgreement.CreatePendingAgreementResponseDocument;
import org.ogf.schemas.graap.wsAgreement.NoncriticalExtensionType;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3c.dom.Element;

/*
 * TODO: Persistence does not work for pending agreements. There is a change of the server API required in order to
 * persist agreements even for pending agreement calls.
 * 
 * => will be done in next major release
 */

/**
 * Default implementation of the agreement factory port type.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AgreementFactoryCapability extends WSAG4JCapability
{

    private static final Logger LOG = Logger.getLogger( AgreementFactoryCapability.class );

    private PersistentAgreementFactory getFactoryPersistence()
    {
        return ( (AgreementFactoryWsResource) getResource() ).getAgreementFactory();
    }

    /**
     * @return the resource property QNames
     */
    public QName[] getPropertyNames()
    {
        return WsagConstants.WSAG_AGREEMENT_FACTORY_PROPERTIES;
    }

    /**
     * 
     * @return the agreement templates exposed by the factory
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public Element[] getTemplate() throws ResourceUnknownFault, ResourceUnavailableFault
    {

        // if the resource router is not initialized we return an empty template array
        if ( !getResource().getResourceManager().hasBeenInitialized()
            || getResource().getResourceManager().hasBeenShutdown() )
        {
            return new Element[] {};
        }

        try
        {

            AgreementTemplateType[] templates = getFactoryPersistence().getTemplates();

            AgreementFactoryPropertiesDocument doc = AgreementFactoryPropertiesDocument.Factory.newInstance();
            doc.addNewAgreementFactoryProperties().setTemplateArray( templates );

            // assure that we convert xml-elements
            return toElementArray( doc.getAgreementFactoryProperties().getTemplateArray() );
        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( Exception e )
        {
            String message = "Illegal state in GetTemplates method. Error: " + e.getMessage();
            LOG.error( message, e );

            throw new ResourceUnavailableFault( message, e );
        }
    }

    /**
     * Creates an agreement offer for further processing from a create agreement request.
     * 
     * @return the new agreement offer
     */
    private AgreementOffer createAgreementOffer( final AgreementType offer,
                                                 final EndpointReferenceType initiatorEPR,
                                                 final NoncriticalExtensionType[] nce )
    {

        return new AgreementOffer()
        {

            private AgreementType nOffer = offer;

            private EndpointReferenceType nInitiatorEPR = initiatorEPR;

            private NoncriticalExtensionType[] nNCE = nce;

            public AgreementContextType getContext()
            {
                return nOffer.getContext();
            }

            public EndpointReferenceType getInitiatorEPR()
            {
                return nInitiatorEPR;
            }

            public String getName()
            {
                return nOffer.getName();
            }

            public NoncriticalExtensionType[] getNoncriticalExtensions()
            {
                return nNCE;
            }

            public TermTreeType getTerms()
            {
                return nOffer.getTerms();
            }

            public void setContext( AgreementContextType context )
            {
                nOffer.setContext( context );
            }

            public void setInitiatorEPR( EndpointReferenceType initiator )
            {
                nInitiatorEPR = initiator;
            }

            public void setName( String name )
            {
                nOffer.setName( name );
            }

            public void setNoncriticalExtensions( NoncriticalExtensionType[] extension )
            {
                nNCE = extension;
            }

            public void setTerms( TermTreeType terms )
            {
                nOffer.setTerms( terms );
            }

            public AgreementType getXMLObject()
            {
                return nOffer;
            }

            public String getAgreementId()
            {
                return nOffer.getAgreementId();
            }

            public void setAgreementId( String id )
            {
                nOffer.setAgreementId( id );
            }

        };
    }

    /**
     * Creates an agreement for an offer.
     * 
     * @param inputDoc
     *            the agreement offer
     * 
     * @return the endpoint of the new created agreement
     * 
     * @throws AgreementFactoryFault
     *             an AgreementFactoryFault indicates that the agreement offer was rejected
     * 
     * @throws ResourceUnknownFault
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced resource is unavailable
     */
    public CreateAgreementOutputType createAgreement( CreateAgreementInputDocument inputDoc )
        throws AgreementFactoryFault, ResourceUnknownFault, ResourceUnavailableFault
    {
        final CreateAgreementInputType input = inputDoc.getCreateAgreementInput();

        AgreementOffer offer =
            createAgreementOffer( input.getAgreementOffer(), input.getInitiatorAgreementEPR(),
                                  input.getNoncriticalExtensionArray() );

        Agreement createdAgreement = null;

        try
        {
            createdAgreement = getFactoryPersistence().createAgreement( offer );

        }
        catch ( AgreementFactoryException e )
        {
            throw new AgreementFactoryFault( e );
        }
        catch ( Exception e )
        {
            String message = "Illegal state in CreateAgreement method. Error: " + e.getMessage();
            LOG.error( message );

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "failed to process create agreement request" );
                LOG.debug( inputDoc.xmlText( new XmlOptions().setSavePrettyPrint() ) );
                LOG.debug( e );
            }

            throw new ResourceUnavailableFault( message, e );
        }

        EndpointReferenceType epr = addNewAgreement( createdAgreement );

        CreateAgreementResponseDocument response = CreateAgreementResponseDocument.Factory.newInstance();
        response.addNewCreateAgreementResponse().setCreatedAgreementEPR( epr );
        return response.getCreateAgreementResponse();
    }

    /**
     * Creates an agreement for the given offer. The decision of agreement acceptance or rejection is delayed
     * to a later time. As long the decision was not made, the returned agreement is in the PENDING state.
     * 
     * @param paInput
     *            the agreement offer
     * 
     * @return the endpoint to the pending agreement
     * 
     * @throws AgreementFactoryFault
     *             an error occurred in the agreement creation process
     * 
     * 
     * @throws ResourceUnknownFault
     *             the referenced agreement factory is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced agreement factory is unavailable
     */
    public CreatePendingAgreementOutputType createPendingAgreement( CreatePendingAgreementInputDocument paInput )
        throws AgreementFactoryFault, ResourceUnknownFault, ResourceUnavailableFault
    {
        final CreatePendingAgreementInputType input = paInput.getCreatePendingAgreementInput();

        AgreementOffer offer =
            createAgreementOffer( input.getAgreementOffer(), input.getInitiatorAgreementEPR(),
                                  input.getNoncriticalExtensionArray() );

        Agreement createdAgreement = null;

        try
        {

            AgreementAcceptanceClient client = null;
            EndpointReferenceType acceptanceEPR =
                paInput.getCreatePendingAgreementInput().getAgreementAcceptanceEPR();

            if ( acceptanceEPR != null )
            {
                client =
                    AgreementAcceptanceLocatorImpl.getAcceptanceClient( acceptanceEPR,
                                                                        WsagEngine.getLoginContext() );
            }

            createdAgreement = createPendingAgreementImpl( offer, client );
        }
        catch ( AgreementFactoryException e )
        {
            throw new AgreementFactoryFault( e );
        }
        catch ( ResourceUnknownException e )
        {
            throw new ResourceUnknownFault( e );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( Exception e )
        {
            String message = "Illegal state in CreateAgreement method. Error: " + e.getMessage();
            LOG.error( message );

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( e );
            }

            throw new ResourceUnavailableFault( message, e );
        }

        EndpointReferenceType epr = addNewAgreement( createdAgreement );

        CreatePendingAgreementResponseDocument response =
            CreatePendingAgreementResponseDocument.Factory.newInstance();
        response.addNewCreatePendingAgreementResponse().setCreatedAgreementEPR( epr );
        return response.getCreatePendingAgreementResponse();
    }

    /**
     * Creates a new Agreement WS-Resource and returns the EPR.
     * 
     * @param agreement
     *            the agreement to add
     * 
     * @return the ERP of the added agreement
     * 
     * @throws AgreementFactoryFault
     *             an error occurred in the agreement creation process
     * 
     * 
     * @throws ResourceUnknownFault
     *             the referenced agreement factory is unknown
     * 
     * @throws ResourceUnavailableFault
     *             the referenced agreement factory is unavailable
     */
    protected EndpointReferenceType addNewAgreement( Agreement agreement )
        throws AgreementFactoryFault, ResourceUnknownFault, ResourceUnavailableFault
    {
        try
        {
            //
            // create a new agreement WS resource
            //
            Resource agreementInstance = getResource().getResourceManager().createResource( "Agreement" );

            //
            // lookup and set the WSAG4J session object
            //
            XmlObject[] sessionDocuments =
                agreement.getContext().selectChildren( WSAG4JSessionDocument.type.getDocumentElementName() );
            if ( sessionDocuments.length > 0 )
            {
                if ( sessionDocuments.length > 1 )
                {
                    String message =
                        "Found multiple wsag4j session documents in agreement context. "
                            + "Using the first, ignoring the rest.";
                    LOG.warn( message );
                }

                WSAG4JSessionType sessionToken = (WSAG4JSessionType) sessionDocuments[0];

                WsagSession session = WsagSessionManager.getSession( sessionToken.getSessionID() );
                if ( session != null )
                {
                    ( (AgreementWsResource) agreementInstance ).setSession( session );
                }
                else
                {
                    LOG.error( "WSAG4J session with id [" + sessionToken.getSessionID() + "] not found." );
                }
            }

            //
            // initialize the agreement WS resource object, initialize the WS resources,
            // and add it to the resource manager
            //
            EndpointReference museFactoryEPR = getResource().getEndpointReference();
            EndpointReferenceType factoryEPR = XmlUtils.convertMuseEPRToEndpoint( museFactoryEPR );

            ( (AgreementWsResource) agreementInstance ).setFactoryEPR( factoryEPR );
            ( (AgreementWsResource) agreementInstance ).setAgreement( agreement );

            agreementInstance.initialize();
            getResource().getResourceManager().addResource( agreementInstance.getEndpointReference(),
                                                            agreementInstance );

            //
            // add the new created agreement to the WSAG4J persistence layer
            //
            EndpointReference museAgreementEPR = agreementInstance.getEndpointReference();
            EndpointReferenceType agreementEPR = XmlUtils.convertMuseEPRToEndpoint( museAgreementEPR );
            // getFactoryPersistence().addAgreement(agreement, agreementEPR);

            //
            // store the agreement <-> epr kombination
            //
            AgreementEprContainer agreementEprContainer = new AgreementEprContainer();
            agreementEprContainer.setAgreementId( agreement.getAgreementId() );
            agreementEprContainer.setEpr( agreementEPR );
            agreementEprContainer.setAgreementFactoryId( getFactoryPersistence().getResourceId() );

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "Create a new AgreementEprContainer. Try to persist it." );
            }

            EntityManager em = EmfRegistry.getEntityManager();
            em.getTransaction().begin();

            try
            {
                em.persist( agreementEprContainer );
                em.getTransaction().commit();
            }
            catch ( RollbackException ex )
            {
                LOG.error( "Could not persist container storing the agreement id and EPR.", ex );
                em.getTransaction().rollback();
            }
            finally
            {
                em.close();
            }

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "AgreementEprContainer persisted." );
            }

            //
            // store the creator of the agreement instance and notify observers of change of persistent properties
            //
            Subject subject = (Subject) WsagEngine.getWsagMessageContext().get( SecurityConstants.AUTHENTICATED_USER );
            ( (AgreementWsResource) agreementInstance ).setSubject( subject );
            agreement.getAgreementInstance().notifyObservers();
            
            //
            // return the EPR of the new agreement instance
            //
            EndpointReferenceType epr =
                XmlUtils.convertMuseEPRToEndpoint( agreementInstance.getEndpointReference() );

            return epr;
        }
        catch ( Exception ex )
        {
            // This should not happen, but if it does we have to throw an
            // exception, since we can't handle this here
            throw new IllegalStateException(
                                             "An internal error occured while adding a new agreement. Message: "
                                                 + ex.getMessage() );
        }
    }

    /**
     * Creates a new pending agreement instance. If an acceptance EPR is provided, the initiator is notified
     * of the result.
     * 
     * @param offer
     *            the agreement offer
     * @param acceptanceClient
     *            the agreement acceptance client
     * 
     * @return the created agreement instance
     * 
     * @throws AgreementFactoryException
     *             indicates that the agreement offer was rejected by the factory
     * 
     * @throws ResourceUnknownException
     *             the referenced resource is unknown
     * 
     * @throws ResourceUnavailableException
     *             the referenced resource is unavailable
     */
    protected Agreement createPendingAgreementImpl( AgreementOffer offer,
                                                    AgreementAcceptanceClient acceptanceClient )
        throws AgreementFactoryException, ResourceUnknownException, ResourceUnavailableException
    {
        PendingAgreementFacade agreement =
            new PendingAgreementFacade( offer, getFactoryPersistence(), acceptanceClient );
        Thread runner = new Thread( agreement );
        runner.start();
        return agreement;
    }

}
