/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import org.apache.muse.ws.resource.basefaults.BaseFault;
import org.apache.muse.ws.resource.faults.ResourceUnavailableFault;
import org.apache.muse.ws.resource.properties.impl.SimpleResourcePropertyCollection;
import org.apache.ws.security.WSSecurityException;
import org.ogf.schemas.graap.wsAgreement.AgreementPropertiesDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementPropertiesType;
import org.w3c.dom.Element;

/**
 * Implementation for retrieving the agreement resource properties document.
 * 
 * @author owaeld
 * 
 */
public class AgreementPropertiesCollection extends SimpleResourcePropertyCollection
{

    private AgreementWsResource agreementResource;

    /**
     * Creates a new agreement resource properties collection for the given agreement. The collection returns
     * the complete resource properties document for this agreement.
     * 
     * @param agreementResource
     *            the agreement resource where this collection belongs to
     */
    public AgreementPropertiesCollection( AgreementWsResource agreementResource )
    {
        this.agreementResource = agreementResource;
    }

    /**
     * Returns the agreement properties document of this agreement instance.
     * 
     * @return the resource properties document of the agreement
     * 
     * @throws BaseFault
     *             {@inheritDoc}
     * 
     * @see Agreement#getAgreementInstance()
     * @see org.ogf.graap.wsag.api.types.AbstractAgreementType#getXMLObject()
     * @see org.apache.muse.ws.resource.properties.impl.SimpleResourcePropertyCollection#getResourcePropertyDocument()
     */
    @Override
    public synchronized Element getResourcePropertyDocument() throws BaseFault
    {
        try
        {
            AgreementPropertiesType properties =
                (AgreementPropertiesType) agreementResource.getAgreement().getAgreementInstance().getXMLObject();
            AgreementPropertiesDocument doc = AgreementPropertiesDocument.Factory.newInstance();
            doc.addNewAgreementProperties().set( properties );

            return (Element) doc.getAgreementProperties().getDomNode();
        }
        catch ( WSSecurityException e )
        {
            throw new ResourceUnavailableFault( e );
        }
    }

}
