/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import java.security.cert.X509Certificate;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.Set;

import javax.security.auth.Subject;
import javax.xml.namespace.QName;

import org.apache.muse.ws.resource.properties.ResourcePropertyCollection;
import org.apache.ws.security.WSSecurityException;
import org.apache.xmlbeans.XmlString;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.security.core.SecurityConstants;
import org.ogf.graap.wsag.server.api.WsagSession;
import org.ogf.graap.wsag.server.engine.WsagEngine;
import org.ogf.graap.wsag.wsrf.AbstractWsResource;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * Default implementation of an agreement WSRF resource. Each agreement WSRF resource has an associated
 * agreement instance, that implements for example agreement monitoring capabilities.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AgreementWsResource extends AbstractWsResource
{

    /**
     * 
     */
    private static final String WSAG4J_SECURITY_DN = "wsag4j.security.dn";

    private Agreement agreement = null;

    private WsagSession session = null;

    private EndpointReferenceType factoryEPR = null;

    private String subjectDN = null;

    private boolean subjectInitiatlized = false;

    /**
     * Initializes the subject DN with the DN of the entity that created the agreement instance.
     */
    private void initializeSubjectDN()
    {
        if ( subjectInitiatlized )
        {
            return;
        }

        try
        {
            XmlString xmlObject =
                (XmlString) agreement.getAgreementInstance().getExecutionContext().get( WSAG4J_SECURITY_DN );
            subjectDN = xmlObject.getStringValue();
            subjectInitiatlized = true;
        }
        catch ( Exception e )
        {
            throw new RuntimeException( e );
        }
    }

    /**
     * Sets the subject of the entity that created the agreement instance.
     * 
     * @param subject
     *            the subject to set
     */
    public void setSubject( Subject subject )
    {
        try
        {
            //
            // empty dn is treated as not authenticated
            //
            subjectDN = "";

            try
            {
                subjectDN = resolveSubjectDN( subject );
            }
            catch ( Exception e )
            {
                throw new RuntimeException( "failed to resolve authenticated user DN", e );
            }

            //
            // store the dn in the agreement properties, so it is persisted with the agreement
            //
            agreement.getAgreementInstance().getExecutionContext()
                     .put( WSAG4J_SECURITY_DN, XmlString.Factory.newValue( subjectDN ) );
        }
        catch ( Exception e )
        {
            throw new RuntimeException( e );
        }
    }

    /**
     * @param subject
     */
    private String resolveSubjectDN( Subject subject )
    {
        //
        // if a subject is provided, we extract the dn from the cert used four authentication
        //
        if ( subject != null )
        {
            //
            // get the first public credential
            //
            Set<X509Certificate> cretentials = subject.getPublicCredentials( X509Certificate.class );
            for ( Iterator<X509Certificate> iterator = cretentials.iterator(); iterator.hasNext(); )
            {
                X509Certificate cert = iterator.next();
                return cert.getSubjectDN().toString();
            }
        }

        return null;
    }

    /**
     * @return the associated agreement instance
     * 
     * @throws WSSecurityException
     *             indicates that the user that tries to access the agreement instance is not the one that
     *             created this instance
     */
    public Agreement getAgreement() throws WSSecurityException
    {
        //
        // check that the user accessing the agreement instance is the same that
        // initially created that instance
        //
        Subject currentUser =
            (Subject) WsagEngine.getWsagMessageContext().get( SecurityConstants.AUTHENTICATED_USER );

        //
        // access to the the agreement instance is permitted if one of the following conditions is fulfilled:
        //
        // 1. the resource is currently initializing
        // 2. creator or current user is null and anonymous access is allowed (testing)
        // 3. creator and current user are equal
        //
        if ( !isInitialized() )
        {
            return agreement;
        }

        initializeSubjectDN();

        if ( ( currentUser == null ) || ( subjectDN == null ) )
        {
            if ( WsagEngine.isAllowAnonymousAccess() )
            {
                return agreement;
            }
        }
        else if ( subjectDN.equals( resolveSubjectDN( currentUser ) ) )
        {
            return agreement;
        }

        //
        // if the authenticated user does not math the creator of the agreement throw an exception
        //
        String message =
            "The authenticated user does not match the creator of the agreement instance."
                + "\nauthenticated: {0}\nexpected: {1}";
        throw new WSSecurityException( MessageFormat.format( message, currentUser, subjectDN ) );
    }

    /**
     * 
     * @param agreement
     *            the associated agreement instance
     */
    public void setAgreement( Agreement agreement )
    {
        this.agreement = agreement;
    }

    /**
     * 
     * @param session
     *            the agreement session
     */
    public void setSession( WsagSession session )
    {
        this.session = session;
    }

    /**
     * 
     * @return the agreement session
     */
    public WsagSession getSession()
    {
        return session;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public QName getInterfaceName()
    {
        return WsagConstants.WSAG_AGREEMENT_QNAME;
    }

    /**
     * @return the factoryEPR
     */
    public EndpointReferenceType getFactoryEPR()
    {
        return factoryEPR;
    }

    /**
     * @param factoryEPR
     *            the factoryEPR to set
     */
    public void setFactoryEPR( EndpointReferenceType factoryEPR )
    {
        this.factoryEPR = factoryEPR;
    }

    /**
     * Returns an {@link AgreementPropertiesCollection} for retrieving the
     * {@link org.ogf.schemas.graap.wsAgreement.AgreementPropertiesDocument} via the WSRF
     * GetResourcePropertyDocument method.
     * 
     * @return the agreement resource property collection
     * 
     * @see AgreementPropertiesCollection
     * @see org.apache.muse.ws.resource.impl.SimpleWsResource#createPropertyCollection()
     */
    @Override
    protected ResourcePropertyCollection createPropertyCollection()
    {
        return new AgreementPropertiesCollection( this );
    }
}
