/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.faults.ResourceUnavailableFault;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.ogf.graap.wsag.api.Negotiation;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.exceptions.NegotiationException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.WSAgreementException;
import org.ogf.graap.wsag.wsrf.WSAG4JCapability;
import org.ogf.schemas.graap.wsAgreement.AgreementFactoryPropertiesDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationOfferType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.AdvertiseInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.AdvertiseInputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.AdvertiseOutputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiateInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiateInputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiateOutputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.NegotiationPropertiesDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.TerminateInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.TerminateResponseDocument;
import org.w3c.dom.Element;

/**
 * Default implementation of the WS-Agreement Negotiation port type.
 * 
 * @author hrasheed
 * 
 */
public class NegotiationCapability extends WSAG4JCapability
{

    private static final Logger LOG = Logger.getLogger( NegotiationCapability.class );

    /**
     * @return the resource property QNames
     */
    public QName[] getPropertyNames()
    {
        return WsagConstants.WSAG_NEGOTIATION_PROPERTIES;
    }

    private Negotiation getNegotiation()
    {
        return ( (NegotiationWsResource) getResource() ).getNegotiation();
    }

    /**
     * Returns the context of this negotiation instance.
     * 
     * @return the context of this negotiation instance
     * 
     * @throws ResourceUnavailableFault
     *             the negotiation resource is not available
     */
    public Element getNegotiationContext() throws ResourceUnavailableFault
    {

        try
        {

            NegotiationContextType context = getNegotiation().getNegotiationContext();

            if ( context == null )
            {
                context = NegotiationContextDocument.Factory.newInstance().addNewNegotiationContext();
                Calendar expire = new GregorianCalendar();
                expire.add( Calendar.DAY_OF_YEAR, 1 );
                context.setExpirationTime( expire );
            }

            NegotiationPropertiesDocument propsDoc = NegotiationPropertiesDocument.Factory.newInstance();
            propsDoc.addNewNegotiationProperties().setNegotiationContext( context );

            // NegotiationContextDocument _contextDoc = NegotiationContextDocument.Factory.newInstance();
            // _contextDoc.setNegotiationContext(context);

            return toElement( propsDoc.getNegotiationProperties().getNegotiationContext() );
        }
        catch ( Exception e )
        {
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( e );
            }
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * Returns the negotiable templates supported by this negotiation instance.
     * 
     * @return the negotiable templates
     * 
     * @throws ResourceUnavailableFault
     *             the negotiation resource is not available
     */
    public Element[] getNegotiableTemplate() throws ResourceUnavailableFault
    {
        // if the resource router is not initialized we return an empty template array
        if ( !getResource().getResourceManager().hasBeenInitialized()
            || getResource().getResourceManager().hasBeenShutdown() )
        {
            return new Element[] {};
        }

        try
        {

            AgreementTemplateType[] templates = getNegotiation().getNegotiableTemplates();

            AgreementFactoryPropertiesDocument doc = AgreementFactoryPropertiesDocument.Factory.newInstance();
            doc.addNewAgreementFactoryProperties().setTemplateArray( templates );

            // assure that we convert xml-elements
            return toElementArray( doc.getAgreementFactoryProperties().getTemplateArray() );
        }
        catch ( ResourceUnavailableException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( WSAgreementException e )
        {
            throw new ResourceUnavailableFault( e );
        }
        catch ( Exception e )
        {
            LOG.error( "Luke, I am your father.", e );

            String message = "Illegal state in GetTemplates method. Error: " + e.getMessage();
            LOG.error( message );

            throw new ResourceUnavailableFault( message, e );
        }
    }

    /**
     * Returns a history of exchanged negotiation offers.
     * 
     * @return the negotiation offers history
     * 
     * @throws ResourceUnavailableFault
     *             the negotiation resource is not available
     * 
     * @todo not implemented yet
     */
    public Element[] getNegotiationOffer() throws ResourceUnavailableFault
    {

        try
        {

            NegotiationOfferType[] currentOffers = getNegotiation().getNegotiationOffers();

            if ( currentOffers == null )
            {
                currentOffers = new NegotiationOfferType[0];
            }

            /*
             * NegotiationOfferDocument[] negotiated_offers = new
             * NegotiationOfferDocument[current_offers.length]; for (int i = 0; i < negotiated_offers.length;
             * i++) { negotiated_offers[i] = NegotiationOfferDocument.Factory.newInstance();
             * negotiated_offers[i].setNegotiationOffer(current_offers[i]); }
             */

            NegotiationPropertiesDocument propsDoc = NegotiationPropertiesDocument.Factory.newInstance();
            propsDoc.addNewNegotiationProperties().setNegotiationOfferArray( currentOffers );

            return toElementArray( propsDoc.getNegotiationProperties().getNegotiationOfferArray() );
        }
        catch ( Exception e )
        {
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( e );
            }
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * Negotiates a set of offers and creates c set of counter offers.
     * 
     * @param in
     *            the input document
     * 
     * @return the response document
     * 
     * @throws SoapFault
     *             generic SOAP fault
     */
    public Element negotiate( NegotiateInputDocument in ) throws SoapFault
    {

        try
        {

            NegotiateInputType input = in.getNegotiateInput();
            NegotiationOfferType[] quotes = input.getNegotiationOfferArray();
            XmlObject[] ncExtensions = new XmlObject[0];

            NegotiationOfferType[] negotiatedOffers = getNegotiation().negotiate( quotes, ncExtensions );

            if ( negotiatedOffers == null )
            {
                negotiatedOffers = new NegotiationOfferType[0];
            }

            NegotiateOutputDocument output = NegotiateOutputDocument.Factory.newInstance();
            output.addNewNegotiateOutput().setNegotiationCounterOfferArray( negotiatedOffers );

            return toElement( output );
        }
        catch ( NegotiationException e )
        {
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( e );
            }
            throw new SoapFault( e );
        }
        catch ( Exception e )
        {
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "failed to process negotiation request" );
                LOG.debug( in.xmlText( new XmlOptions().setSavePrettyPrint() ) );
                LOG.debug( e );
            }
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * Advertises a set of negotiation offers to a negotiation participator.
     * 
     * @param in
     *            the input document
     * 
     * @return the response document
     * 
     * @throws SoapFault
     *             generic SOAP fault
     */
    public Element advertise( AdvertiseInputDocument in ) throws SoapFault
    {

        try
        {

            AdvertiseInputType input = in.getAdvertiseInput();

            NegotiationOfferType[] quotes = input.getNegotiationOfferArray();
            XmlObject[] ncExtensions = new XmlObject[0];

            getNegotiation().advertise( quotes, ncExtensions );

            AdvertiseOutputDocument output = AdvertiseOutputDocument.Factory.newInstance();
            output.addNewAdvertiseOutput();

            return toElement( output );
        }
        catch ( NegotiationException e )
        {
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( e );
            }
            throw new SoapFault( e );
        }
        catch ( Exception e )
        {
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( e );
            }
            throw new ResourceUnavailableFault( e );
        }
    }

    /**
     * Terminates a negotiation process.
     * 
     * @param in
     *            the termination reason
     * 
     * @return the termination response document
     * 
     * @throws SoapFault
     *             generic SOAP fault
     */
    public Element terminate( TerminateInputDocument in ) throws SoapFault
    {

        try
        {

            getNegotiation().terminate();

            TerminateResponseDocument output = TerminateResponseDocument.Factory.newInstance();
            output.addNewTerminateResponse();

            return toElement( output );
        }
        catch ( Exception e )
        {
            throw new SoapFault( e );
        }
    }
}
