/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.core.Resource;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.faults.ResourceUnavailableFault;
import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.api.Negotiation;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.graap.wsag.wsrf.WSAG4JCapability;
import org.ogf.graap.wsag.wsrf.XmlUtils;
import org.ogf.graap.wsag.wsrf.faults.AgreementFactoryFault;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.InitiateNegotiationInputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.InitiateNegotiationInputType;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.InitiateNegotiationOutputDocument;
import org.ogf.schemas.graap.wsAgreement.negotiation.protocol.InitiateNegotiationOutputType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * @author hrasheed
 * 
 */
public class NegotiationFactoryCapability extends WSAG4JCapability
{

    private static final Logger LOG = Logger.getLogger( NegotiationFactoryCapability.class );

    private PersistentAgreementFactory getFactoryPersistence()
    {
        return ( (AgreementFactoryWsResource) getResource() ).getAgreementFactory();
    }

    /**
     * @return the resource property QNames
     */
    public QName[] getPropertyNames()
    {
        return new QName[0];
    }

    /**
     * Creates a new negotiation instance.
     * 
     * @param initiateInput
     *            the negotiation context + extensions
     * 
     * @return the initiate negotiation response document
     * 
     * @throws SoapFault
     *             generic SOAP fault
     */
    public InitiateNegotiationOutputType initiateNegotiation( InitiateNegotiationInputDocument initiateInput )
        throws SoapFault
    {

        InitiateNegotiationInputType input = initiateInput.getInitiateNegotiationInput();

        NegotiationContextType context = input.getNegotiationContext();
        XmlObject[] ncExtensions = input.getNoncriticalExtensionArray();
        XmlObject[] cExtensions = new XmlObject[0];
        Map<String, Object> environment = new HashMap<String, Object>();

        Negotiation createdNegotiation = null;

        try
        {
            //
            // initialize the negotiation process
            //
            createdNegotiation =
                getFactoryPersistence().initiateNegotiation( context, cExtensions, ncExtensions, environment );
        }
        catch ( NegotiationFactoryException e )
        {
            throw new AgreementFactoryFault( e );
        }
        catch ( Exception e )
        {
            String message = "Illegal state in initiateNegotiation method. Error: " + e.getMessage();
            LOG.error( message );
            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( e );
            }
            throw new ResourceUnavailableFault( message, e );
        }

        EndpointReferenceType epr = addNewNegotiationWsResource( createdNegotiation );

        InitiateNegotiationOutputDocument output = InitiateNegotiationOutputDocument.Factory.newInstance();

        output.addNewInitiateNegotiationOutput().setCreatedNegotiationEPR( epr );

        return output.getInitiateNegotiationOutput();
    }

    /**
     * Creates a new Negotiation WS-Resource and returns the EPR.
     * 
     * @param negotiation
     *            the negotiation instance for which the WS resource is created
     * 
     * @return the endpoint of the negotiation resource
     * 
     * @throws AgreementFactoryFault
     *             an error occurred while creating the WS resource
     */
    protected EndpointReferenceType addNewNegotiationWsResource( Negotiation negotiation )
        throws AgreementFactoryFault
    {

        try
        {
            //
            // create a new negotiation WS resource
            //
            Resource negotiationInstance = getResource().getResourceManager().createResource( "Negotiation" );

            //
            // initialize the negotiation WS resource object, initialize the WS resources,
            // and add it to the resource manager
            //
            EndpointReference museFactoryEPR = getResource().getEndpointReference();
            EndpointReferenceType factoryEPR = XmlUtils.convertMuseEPRToEndpoint( museFactoryEPR );

            ( (NegotiationWsResource) negotiationInstance ).setFactoryEPR( factoryEPR );
            ( (NegotiationWsResource) negotiationInstance ).setNegotiation( negotiation );

            negotiationInstance.initialize();
            getResource().getResourceManager().addResource( negotiationInstance.getEndpointReference(),
                                                            negotiationInstance );

            //
            // return the EPR of the new agreement instance
            //
            EndpointReferenceType epr =
                XmlUtils.convertMuseEPRToEndpoint( negotiationInstance.getEndpointReference() );

            return epr;
        }
        catch ( Exception ex )
        {
            throw new AgreementFactoryFault(
                                             "An internal error occured while adding a new negotiation. Message: "
                                                 + ex.getMessage() );
        }
    }

}
