/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.types.AbstractAgreementType;
import org.ogf.graap.wsag.wsrf.AgreementAcceptanceClient;
import org.ogf.schemas.graap.wsAgreement.AgreementContextType;
import org.ogf.schemas.graap.wsAgreement.AgreementStateDefinition;
import org.ogf.schemas.graap.wsAgreement.AgreementStateType;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateType;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateType;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputType;

/**
 * PendingAgreementFacade
 * 
 * @author Oliver Waeldrich
 * 
 */
public class PendingAgreementFacade implements Agreement, Runnable
{

    private static final Logger LOG = Logger.getLogger( PendingAgreementFacade.class );

    private Agreement agreement;

    private AgreementFactory factory;

    private AgreementOffer offer;

    private AgreementAcceptanceClient client;

    private TerminateInputType reason;

    /**
     * Implementation of a pending agreement facade. The agreement creation is started in a separate process.
     * As long as the agreement creation is in progress, the facades represents the pending agreement. If the
     * agreement creation is successful, the pending agreement is replaced with the real one created by the
     * WSAG4J engine. This is an implementation of the state pattern (GOF).
     * 
     * @param offer
     *            the agreement offer
     * @param factory
     *            the agreement factory
     * @param client
     *            the agreement acceptance client
     */
    public PendingAgreementFacade( AgreementOffer offer, AgreementFactory factory,
                                   AgreementAcceptanceClient client )
    {
        this.factory = factory;
        this.offer = offer;
        this.client = client;
        this.agreement = new PendingAgreementImpl( offer );

        this.agreement.getState().setState( AgreementStateDefinition.PENDING );
    }

    /**
     * {@inheritDoc}
     * 
     * @see java.lang.Runnable#run()
     */
    public void run()
    {
        try
        {
            Agreement newAgreement = factory.createAgreement( offer );

            if ( agreement.getState().getState() == AgreementStateDefinition.PENDING_AND_TERMINATING )
            {
                try
                {
                    agreement = newAgreement;
                    agreement.terminate( reason );
                }
                catch ( Exception e )
                {
                    // must not throw any exception
                    LOG.error( e.getMessage() );
                }
            }
            else
            {
                agreement = newAgreement;
                acceptAgreement();
            }
        }
        catch ( Exception e )
        {
            if ( LOG.isInfoEnabled() )
            {
                LOG.info( "Error while creating pending agreement. Cause: " + e.getMessage() );
            }

            rejectAgreement();
        }
    }

    private void rejectAgreement()
    {
        try
        {
            agreement.getState().setState( AgreementStateDefinition.REJECTED );

            if ( client != null )
            {
                client.reject();
            }
        }
        catch ( Exception ex )
        {
            // local call to the pending agreement
            LOG.error( ex.getMessage() );
        }
    }

    private void acceptAgreement()
    {
        try
        {
            if ( client != null )
            {
                client.accept();
            }
        }
        catch ( Exception ex )
        {
            // local call to the pending agreement
            LOG.error( ex.getMessage() );
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getAgreementId()
    {
        return agreement.getAgreementId();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementContextType getContext()
    {
        return agreement.getContext();
    }

    /**
     * {@inheritDoc}
     */
    public GuaranteeTermStateType[] getGuaranteeTermStates()
    {
        return agreement.getGuaranteeTermStates();
    }

    /**
     * {@inheritDoc}
     */
    public String getName()
    {
        return agreement.getName();
    }

    /**
     * {@inheritDoc}
     */
    public ServiceTermStateType[] getServiceTermStates()
    {
        return agreement.getServiceTermStates();
    }

    /**
     * {@inheritDoc}
     */
    public AgreementStateType getState()
    {
        return agreement.getState();
    }

    /**
     * {@inheritDoc}
     */
    public TermTreeType getTerms()
    {
        return agreement.getTerms();
    }

    /**
     * {@inheritDoc}
     */
    public void terminate( TerminateInputType terminateReason )
    {
        this.reason = terminateReason;
        agreement.terminate( terminateReason );
    }

    /**
     * {@inheritDoc}
     */
    public AbstractAgreementType getAgreementInstance()
    {
        return agreement.getAgreementInstance();
    }
}
