/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import java.util.Properties;

import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.xmlbeans.XmlException;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.security.ISecurityProperties;
import org.ogf.graap.wsag.client.remote.WsrfResourceClient;
import org.ogf.graap.wsag.wsrf.AgreementAcceptanceClient;
import org.ogf.schemas.graap.wsAgreement.AcceptAgreementResponseDocument;
import org.ogf.schemas.graap.wsAgreement.RejectAgreementResponseDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3c.dom.Element;

/**
 * Default implementation of an agreement acceptance client.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class RemoteAgreementAcceptanceClientImpl implements AgreementAcceptanceClient
{

    private WsrfResourceClient client;

    /**
     * Creates a new acceptance client for the given acceptance EPR.
     * 
     * @param epr
     *            the acceptance EPR
     * @param securityProperties
     *            the client security properties
     */
    public RemoteAgreementAcceptanceClientImpl( EndpointReferenceType epr,
                                                ISecurityProperties securityProperties )
    {
        this( new WsrfResourceClient( epr, securityProperties ) );
    }

    private RemoteAgreementAcceptanceClientImpl( WsrfResourceClient client )
    {
        this.client = client;
    }

    /**
     * {@inheritDoc}
     */
    public void accept() throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            Element inputType = XmlUtils.createElement( WsagConstants.WSAG_ACCEPT_AGREEMENT_INPUT_QNAME );
            Element body = XmlUtils.createElement( WsagConstants.WSAG_ACCEPT_AGREEMENT_QNAME );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = client.invoke( WsagConstants.WSAG_ACCEPT_AGREEMENT_ACTION, body );

            AcceptAgreementResponseDocument respDoc =
                AcceptAgreementResponseDocument.Factory.parse( response );
            respDoc.getAcceptAgreementResponse();
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public void reject() throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            Element inputType = XmlUtils.createElement( WsagConstants.WSAG_REJECT_AGREEMENT_INPUT_QNAME );
            Element body = XmlUtils.createElement( WsagConstants.WSAG_REJECT_AGREEMENT_QNAME );
            body.appendChild( body.getOwnerDocument().importNode( inputType, true ) );
            Element response = client.invoke( WsagConstants.WSAG_REJECT_AGREEMENT_ACTION, body );

            RejectAgreementResponseDocument respDoc =
                RejectAgreementResponseDocument.Factory.parse( response );
            respDoc.getRejectAgreementResponse();
        }
        catch ( SoapFault e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( XmlException e )
        {
            throw new ResourceUnavailableException( e );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    /**
     * {@inheritDoc}
     */
    public EndpointReferenceType getEndpoint()
    {
        return client.getEndpoint();
    }

    /**
     * {@inheritDoc}
     */
    public Properties getProperties()
    {
        return client.getProperties();
    }

    /**
     * {@inheritDoc}
     */
    public ISecurityProperties getSecurityProperties()
    {
        return client.getSecurityProperties();
    }

    /**
     * {@inheritDoc}
     */
    public boolean isUsingTrace()
    {
        return client.isUsingTrace();
    }

    /**
     * {@inheritDoc}
     */
    public void setProperties( Properties properties )
    {
        client.setProperties( properties );
    }

    /**
     * {@inheritDoc}
     */
    public void setTrace( boolean trace )
    {
        client.setTrace( trace );
    }

}
