/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.impl;

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.apache.axis2.context.MessageContext;
import org.apache.log4j.Logger;
import org.apache.muse.core.Environment;
import org.apache.muse.core.Resource;
import org.apache.muse.core.ResourceManager;
import org.apache.muse.util.xml.XmlUtils;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.MessageHeaders;
import org.apache.muse.ws.addressing.WsaConstants;
import org.apache.muse.ws.addressing.soap.SoapConstants;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.WsrfConstants;
import org.apache.muse.ws.resource.faults.ResourceUnknownFault;
import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.security.core.server.ServerSecurityHandler;
import org.ogf.graap.wsag.server.api.AgreementAcceptanceFactory;
import org.ogf.graap.wsag.server.api.IAgreementFactory;
import org.ogf.graap.wsag.server.api.IAgreementFactoryContext;
import org.ogf.graap.wsag.server.api.WsagMessageContext;
import org.ogf.graap.wsag.server.engine.WsagEngine;
import org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome;
import org.ogf.graap.wsag.wsrf.persistence.IWsAgreementFactoryHome;
import org.ogf.graap.wsag.wsrf.persistence.WsDatabasePersistentAgreement;
import org.ogf.graap.wsag.wsrf.persistence.WsDatabaseWSAG4JPersistence;
import org.ogf.graap.wsag.wsrf.persistence.WsPersistentAgreementFactory;
import org.ogf.graap.wsag.wsrf.sg.impl.MembershipContentRuleContext;
import org.ogf.graap.wsag4j.types.configuration.HandlerType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * SimpleResourceRouter
 * 
 * @author Oliver Waeldrich
 * 
 */
public class WSAG4JResourceRouter extends org.apache.muse.core.routing.SimpleResourceRouter
{

    private static final Logger LOG = Logger.getLogger( WSAG4JResourceRouter.class );

    private boolean initialized = false;

    private boolean hasBeenShutdown = false;

    private static ThreadLocal<ServerSecurityHandler[]> securityHandlerPool = null;

    /**
     * Initializes the WS Resource Router.
     * 
     * @throws SoapFault
     *             indicates a initialization fault
     * 
     * @see org.apache.muse.core.routing.SimpleResourceRouter#initialize()
     */
    public void initialize() throws SoapFault
    {
        super.initialize();

        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( "Initializing WSAG4JResourceRouter..." );
        }

        synchronized ( this )
        {
            if ( !initialized )
            {
                //
                // initialize security handler pool
                //
                securityHandlerPool = new ThreadLocal<ServerSecurityHandler[]>()
                {
                    //
                    // method is called on first invocation of securityHandlerPool.get(),
                    // at this time the wsrf-configuration was already set.
                    //
                    protected synchronized ServerSecurityHandler[] initialValue()
                    {
                        return getMessageHandler();
                    }

                };

                MembershipContentRuleContext.initializeContext();

                //
                // Load the agreement factories
                //
                IAgreementFactoryHome persistence = WsagEngine.getAgreementFactoryHome();

                //
                // wrap in ws persistence layer instance
                //
                IWsAgreementFactoryHome wsPersistence = new WsDatabaseWSAG4JPersistence( persistence );

                //
                // initialize the persistence layer
                //
                try
                {
                    wsPersistence.initialize();
                }
                catch ( Exception e )
                {
                    LOG.error( "Failed to initialize WSAG4J persistent layer. Error: " + e.getMessage() );
                }

                //
                // load the agreement factories and initialize the WS resources
                //
                WsPersistentAgreementFactory[] wsFactories = new WsPersistentAgreementFactory[0];

                try
                {
                    wsFactories = wsPersistence.list();
                }
                catch ( Exception e )
                {
                    LOG.error( "Failed to load persistent agreement factories. " + e.getMessage() );
                }

                deployPersistentFactories( wsFactories );

                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( "Deployed " + wsFactories.length + " agreement factories." );
                }

                //
                // load the agreement of the factory and initialize the WS resources
                //
                for ( int j = 0; j < wsFactories.length; j++ )
                {
                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "Load and initialize agreements of agreement factory '"
                            + wsFactories[j].getResourceId() + "'." );
                    }

                    try
                    {
                        wsFactories[j].load();
                    }
                    catch ( Exception e )
                    {
                        throw new SoapFault( "Persistent Agreement Factory failed to load agreements.", e );
                    }

                    WsDatabasePersistentAgreement[] wsDatabasePersistentAgreements = wsFactories[j].list();
                    deployPersistentAgreements( wsDatabasePersistentAgreements,
                                                wsFactories[j].getAgreementFactoryEPR() );
                }

                initialized = true;
            }
        }

        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( "Initialization of WSAG4JResourceRouter finished..." );
        }
    }

    private void deployPersistentFactories( WsPersistentAgreementFactory[] factories ) throws SoapFault
    {
        //
        // initialize the agreement factory WS resources and the according registries
        //
        for ( int j = 0; j < factories.length; j++ )
        {

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "Calling deployPersistentFactories() the " + j + "-th time." );
            }

            AgreementFactoryWsResource factory =
                (AgreementFactoryWsResource) getResourceManager().createResource( "AgreementFactory" );

            EndpointReferenceType factoryEPR = factories[j].getAgreementFactoryEPR();

            factory.setAgreementFactory( factories[j] );
            factory.setEndpointReference( factoryEPR );

            //
            // initialize the factory WSRF resource and populate the
            // factory context with an AgreementAcceptanceFactory instance
            // in case the agreement factory implements the IAgreementFactory
            // interface
            //
            factory.initialize();

            AgreementFactory realFactory = factory.getAgreementFactory();

            if ( realFactory instanceof IAgreementFactory )
            {
                IAgreementFactoryContext context = ( (IAgreementFactory) realFactory ).getFactoryContext();

                context.put( AgreementAcceptanceFactory.AGREEMENT_ACCEPTANCE_FACTORY,
                             new AgreementAcceptanceFactoryImpl( factory ) );
            }

            getResourceManager().addResource( factory.getEndpointReference(), factory );

            if ( LOG.isDebugEnabled() )
            {
                String url = factoryEPR.getAddress().getStringValue();
                XmlObject resIdDoc =
                    factoryEPR.getReferenceParameters().selectChildren( WsagConstants.WSAG4J_RESOURCE_ID_QNAME )[0];
                String resId = resIdDoc.getDomNode().getFirstChild().getNodeValue();
                LOG.debug( "Successfully deployed agreement factory. URL: " + url + " Resource ID: " + resId );
                // log.debug(factoryEPR.xmlText(new XmlOptions().setSavePrettyPrint()));
            }

            //
            // load and initialize the associated agreement registry for the factory.
            // The registry will have the same reference parameters as the factory,
            // but is deployed at its own deployment end point.
            //
            Resource registry = getResourceManager().createResource( "AgreementServiceGroup" );

            EndpointReferenceType registryEPR =
                org.ogf.graap.wsag.wsrf.XmlUtils.convertMuseEPRToEndpoint( registry.getEndpointReference() );

            registryEPR.unsetReferenceParameters();
            registryEPR.addNewReferenceParameters().set( factoryEPR.getReferenceParameters() );

            EndpointReference museRegistryEPR =
                org.ogf.graap.wsag.wsrf.XmlUtils.convertEndpointToMuseEPR( registryEPR );
            registry.setEndpointReference( museRegistryEPR );

            registry.initialize();
            getResourceManager().addResource( registry.getEndpointReference(), registry );

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "Successfully deployed agreement registry." );
            }
        }
    }

    private void deployPersistentAgreements( WsDatabasePersistentAgreement[] wsDatabasePersistentAgreements,
                                             EndpointReferenceType factoryEPR ) throws SoapFault
    {
        if ( LOG.isDebugEnabled() )
        {
            LOG.debug( "Try to load " + wsDatabasePersistentAgreements.length
                + " persisted agreements for agreement factory with EPR address '" + factoryEPR.getAddress()
                + "'." );
        }

        for ( int k = 0; k < wsDatabasePersistentAgreements.length; k++ )
        {
            AgreementWsResource agreement =
                (AgreementWsResource) getResourceManager().createResource( "Agreement" );

            WsDatabasePersistentAgreement wsDatabasePersistentAgreement = wsDatabasePersistentAgreements[k];
            if ( wsDatabasePersistentAgreement == null )
            {
                throw new SoapFault( "WS persistent agreement was NULL." );
            }

            EndpointReferenceType agreementEPR = wsDatabasePersistentAgreement.getAgreementEPR();
            agreement.setEndpointReference( agreementEPR );

            agreement.setAgreement( wsDatabasePersistentAgreement.getAgreement() );
            agreement.setFactoryEPR( factoryEPR );

            agreement.initialize();

            getResourceManager().addResource( agreement.getEndpointReference(), agreement );

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "Successfully deployed persisted agreement." );
            }
        }
    }

    /**
     * 
     * Queries the Environment to get the WS-A EPR being targeted by the current request and then does a
     * lookup in the ResourceManager with that EPR. This method is essential to all routing activity. The
     * resource returned is guaranteed to be the resource specified by the client, but it is not guaranteed to
     * support the specified operation.
     * 
     * @return The resource whose EPR maps to the WS-A data in the current request.
     * 
     * @throws SoapFault
     *             <ul>
     *             <li>If the EPR (including any unique reference properties) is not associated with any
     *             resource instance.</li>
     *             </ul>
     * 
     * @see MessageHeaders#getToAddress()
     * @see ResourceManager#getResource(EndpointReference)
     * 
     */
    protected Resource getTargetResource() throws SoapFault
    {
        ResourceManager manager = getResourceManager();
        Environment env = getEnvironment();
        MessageHeaders wsa = env.getAddressingContext();

        if ( wsa == null )
        {
            throw new RuntimeException( "NoAddressingContext" );
        }

        EndpointReference epr = wsa.getToAddress();

        Resource resource = manager.getResource( epr );

        if ( resource == null )
        {
            //
            // construct an error message that has a formatted list of
            // all of the valid EPRs so that users can determine what
            // was wrong with the EPR they provided
            //

            String museNS = "http://ws.apache.org/muse";
            Element noEPR = XmlUtils.createElement( new QName( museNS, "EndpointNotFound" ) );
            Element reqEPR = noEPR.getOwnerDocument().createElementNS( museNS, "RequestedEndpoint" );
            Element avEPR = noEPR.getOwnerDocument().createElementNS( museNS, "AvailableEndpoints" );

            noEPR.appendChild( reqEPR );
            noEPR.appendChild( avEPR );

            reqEPR.appendChild( epr.toXML( reqEPR.getOwnerDocument() ) );

            // might be used for the debug log
            StringBuffer eprNotFound = new StringBuffer( 512 );
            eprNotFound.append( "\n\n" );
            eprNotFound.append( epr );
            eprNotFound.append( '\n' );

            @SuppressWarnings( "unchecked" )
            Iterator<Object> i = manager.getResourceEPRs();
            StringBuffer eprListing = new StringBuffer( 1024 );
            eprListing.append( '\n' );

            while ( i.hasNext() )
            {
                eprListing.append( '\n' );
                eprListing.append( i.next() );
            }

            ResourceUnknownFault fault =
                new ResourceUnknownFault( "There is no resource available at the given EPR:\n " + eprNotFound
                    + ". \nThe existing EPRs hosted by this endpoint are:\n " + eprListing );

            //
            // The BaseFault Implementation is buggy. Therefore we switch to
            // the SOAPFault implementation here, hopefully with more luck.
            //
            SoapFault soapFault = new SoapFault( fault.toXML() );
            Document owner = soapFault.getDetail().getOwnerDocument();

            Element timestamp = owner.createElementNS( WsrfConstants.NAMESPACE_URI, "Timestamp" );
            Element description = owner.createElementNS( WsrfConstants.NAMESPACE_URI, "Description" );

            timestamp.appendChild( XmlUtils.convertToNode( owner, fault.getTimestamp() ) );
            description.appendChild( XmlUtils.convertToNode( owner, "Resource unknown" ) );

            soapFault.setCode( SoapConstants.SENDER_QNAME );
            soapFault.setSubCode( new QName( WsaConstants.NAMESPACE_URI, "DestinationUnreachable", "wsa" ) );
            soapFault.getDetail().appendChild( timestamp );
            soapFault.getDetail().appendChild( description );
            soapFault.getDetail().appendChild( owner.importNode( noEPR, true ) );

            throw soapFault;
        }

        return resource;
    }

    /**
     * Invokes a SOAP request on the resource router.
     * 
     * @param request
     *            the SOAP request as DOM document
     * 
     * @return the SOAP response (this might also be a SOAP error)
     * 
     * @see org.apache.muse.core.routing.SimpleResourceRouter#invoke(org.w3c.dom.Element)
     */
    public Element invoke( Element request )
    {
        //
        // load the configured security handler
        //
        ServerSecurityHandler[] securityHandlers = securityHandlerPool.get();

        //
        // set a new message context before invoking the server engine
        //
        WsagEngine.setWsagMessageContext( new WsagMessageContext() );

        //
        // initially populate the message context
        //
        WsagEngine.getWsagMessageContext().put( WsagMessageContext.AXIS_MESSAGE_CONTEXT,
                                                MessageContext.getCurrentMessageContext() );

        //
        // let the security handler process the message
        // before invoking the server engine
        //
        for ( int i = 0; i < securityHandlers.length; i++ )
        {
            try
            {
                securityHandlers[i].handleRequest( request );
            }
            catch ( Exception e )
            {
                String message =
                    "[" + securityHandlers[i].getClass().getName()
                        + "]: Error while processing the message request headers. (" + e.getMessage() + ")";
                LOG.error( message );

                throw new RuntimeException( message );
            }
        }

        Element result = super.invoke( request );

        //
        // let the security handler process the result message
        // before handing it back to the isolation layer
        //
        for ( int i = 0; i < securityHandlers.length; i++ )
        {
            try
            {
                securityHandlers[i].handleResponse( result );
            }
            catch ( Exception e )
            {
                String message =
                    "[" + securityHandlers[i].getClass().getName()
                        + "]: Error while processing the message response headers. (" + e.getMessage() + ")";
                LOG.error( message );

                throw new RuntimeException( message );
            }
        }

        //
        // cleanup after usage
        //
        WsagEngine.setWsagMessageContext( null );

        //
        // return the response to the isolation layer
        //
        return result;
    }

    /**
     * loads the security handler configured for this WSAG4J engine
     */
    private static ServerSecurityHandler[] getMessageHandler()
    {

        if ( WsagEngine.getWSRFConfiguration().isSetSecurityHandlerChain() )
        {
            HandlerType[] handler =
                WsagEngine.getWSRFConfiguration().getSecurityHandlerChain().getHandlerArray();
            ServerSecurityHandler[] result = new ServerSecurityHandler[handler.length];

            for ( int i = 0; i < handler.length; i++ )
            {
                try
                {
                    @SuppressWarnings( "unchecked" )
                    Class<ServerSecurityHandler> implementation =
                        (Class<ServerSecurityHandler>) Class.forName( handler[i].getImplementationClass() );
                    result[i] = (ServerSecurityHandler) implementation.newInstance();
                }
                catch ( ClassNotFoundException e )
                {
                    throw new RuntimeException( "The implementation class "
                        + handler[i].getImplementationClass() + " for handler " + handler[i].getHandlerName()
                        + " could not be found." );
                }
                catch ( IllegalAccessException e )
                {
                    throw new RuntimeException( "The implementation class "
                        + handler[i].getImplementationClass() + " for handler " + handler[i].getHandlerName()
                        + " could not be accessed. [" + e.getMessage() + "]" );
                }
                catch ( InstantiationException e )
                {
                    throw new RuntimeException( "The implementation class "
                        + handler[i].getImplementationClass() + " for handler " + handler[i].getHandlerName()
                        + " could not be instantiated. [" + e.getMessage() + "]" );
                }
                catch ( ClassCastException e )
                {
                    throw new RuntimeException( "The implementation class "
                        + handler[i].getImplementationClass() + " does not implement the interface "
                        + ServerSecurityHandler.class.getName() + "." );
                }
            }

            return result;
        }
        else
        {
            return new ServerSecurityHandler[0];
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.muse.core.routing.SimpleResourceRouter#shutdown()
     */
    @SuppressWarnings( "unchecked" )
    @Override
    public void shutdown()
    {
        try
        {
            Iterator<EndpointReference> i = getResourceManager().getResourceEPRs();

            while ( i.hasNext() )
            {
                EndpointReference epr = i.next();
                Resource next = getResourceManager().getResource( epr );
                next.shutdown();
                i = getResourceManager().getResourceEPRs();
            }
        }
        catch ( SoapFault e )
        {
            LOG.error( e );
        }

        //
        // prevent meory leak
        //
        if ( securityHandlerPool != null )
        {
            securityHandlerPool.set( null );
            securityHandlerPool = null;
        }

        MembershipContentRuleContext.finalizeContext();

        hasBeenShutdown = true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.apache.muse.core.routing.SimpleResourceRouter#hasBeenShutdown()
     */
    @Override
    public boolean hasBeenShutdown()
    {
        return hasBeenShutdown;
    }
}
