/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.persistence;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.Query;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.server.persistence.EmfRegistry;
import org.ogf.graap.wsag.server.persistence.PersistedResourceException;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * Entity class for the agreement persistence. It is used to build the relationship between an agreement and
 * an EPR. It defines the entity fields (the tables' columns), the named-queries used to access the stored
 * data, and the corresponding getter/setter methods.
 * 
 * @author T.Weuffel
 */
@Entity
@Table( name = "AGREEMENT_EPR_CONTAINER", uniqueConstraints = @UniqueConstraint( columnNames = {
    "agreement_id", "agreement_factory_id" } ) )
@NamedQueries( {
    @NamedQuery( name = "AgreementEprContainer.findAll", query = "SELECT a FROM AgreementEprContainer a" ),
    @NamedQuery(
                    name = "AgreementEprContainer.findAllByAgreementFactoryId",
                    query = "SELECT a FROM AgreementEprContainer a WHERE a.agreementFactoryId = :agreementFactoryId" ),
    @NamedQuery( name = "AgreementEprContainer.findByEpr",
                    query = "SELECT a FROM AgreementEprContainer a WHERE a.epr = :epr" ),
    @NamedQuery( name = "AgreementEprContainer.findByEprAddress",
                    query = "SELECT a FROM AgreementEprContainer a WHERE a.eprAddress = :eprAddress" ),
    @NamedQuery( name = "AgreementEprContainer.findByAgreementId",
                    query = "SELECT a FROM AgreementEprContainer a WHERE a.agreementId = :agreementId" ) } )
public class AgreementEprContainer
{

    @Transient
    private static final int EPR_FIELD_SIZE = 163840;

    // the logger should not be persisted
    @Transient
    private static final Logger LOG = Logger.getLogger( AgreementEprContainer.class );

    // primary-key of the stored entity
    @SuppressWarnings( "unused" )
    @Id
    @Column( name = "id" )
    @GeneratedValue( strategy = GenerationType.TABLE )
    @Basic( optional = false )
    private Integer id;

    // ID of the agreement
    @Column( name = "agreement_id", nullable = false )
    @Basic( optional = false )
    private String agreementId;

    // ID of the agreement factory, which created the agreement
    @Column( name = "agreement_factory_id", nullable = false )
    @Basic( optional = false )
    private String agreementFactoryId;

    // XML document of the EPR
    @Lob
    @Column( name = "epr", nullable = false, length = EPR_FIELD_SIZE )
    @Basic( optional = false )
    private String epr;

    // address of the EPR
    @Column( name = "epr_address", nullable = false )
    @Basic( optional = false )
    private String eprAddress;

    /**
     * The default constructor is required by the JPA2 environment. It is used to instantiate an instance of
     * this entity. All values are then passed to this instance by setter-injection.
     */
    public AgreementEprContainer()
    {
    }

    /**
     * Used to create an instance of this entity ba hand.
     * 
     * @param agreementId
     *            ID of the agreement.
     * @param epr
     *            EPR of the agreement.
     */
    public AgreementEprContainer( String agreementId, EndpointReferenceType epr )
    {
        this.agreementId = agreementId;
        this.setEpr( epr );
    }

    /**
     * 
     * @return the agreement id
     */
    public String getAgreementId()
    {
        return agreementId;
    }

    /**
     * 
     * @param agreementId
     *            the agreement id to set
     */
    public void setAgreementId( String agreementId )
    {
        this.agreementId = agreementId;
    }

    /**
     * Returns the EPR of the persisted resourse.
     * 
     * @return the agreement endpoint reference
     * 
     * @throws PersistedResourceException
     *             indicates that the persisted EPR could not be de-serialized
     */
    public EndpointReferenceType getEpr() throws PersistedResourceException
    {
        try
        {
            EndpointReferenceType theEpr = EndpointReferenceType.Factory.parse( epr );
            return theEpr;
        }
        catch ( Exception e )
        {
            throw new PersistedResourceException( e );
        }
    }

    /**
     * 
     * @param epr
     *            the agreement endpoint reference to set
     */
    public void setEpr( EndpointReferenceType epr )
    {
        this.epr = epr.xmlText();

        assert epr != null;
        assert epr.getAddress() != null;
        this.eprAddress = epr.getAddress().getStringValue();
    }

    /**
     * 
     * @return the agreement factory id
     */
    public String getAgreementFactoryId()
    {
        return agreementFactoryId;
    }

    /**
     * 
     * @param agreementFactoryId
     *            the agreement factory id to set
     */
    public void setAgreementFactoryId( String agreementFactoryId )
    {
        this.agreementFactoryId = agreementFactoryId;
    }

    /**
     * @return the address of the EPR
     */
    public String getEprAddress()
    {
        return eprAddress;
    }

    /**
     * 
     * @param eprAddress
     *            the address of the EPR to set
     */
    public void setEprAddress( String eprAddress )
    {
        this.eprAddress = eprAddress;
    }

    // /////////////////////////////////////////////////////////////////////////
    // /// Methods for EPR lookup and manipulation /////
    // /////////////////////////////////////////////////////////////////////////

    /**
     * Finds the EPR for the agreement with the given id.
     * 
     * @param agreementId
     *            identifies the agreement for which the EPR is resolved
     * 
     * @return the agreement EPR
     * 
     * @throws Exception
     *             an exception if the agreement was not found or an other error occurred
     */
    public static EndpointReferenceType findEPRForAgreement( String agreementId ) throws Exception
    {
        LOG.trace( "AgreementEprContainer -> findEPRForAgreement(String agreementId)" );

        // try to find the agreement EPR
        EntityManager em = EmfRegistry.getEntityManager();
        AgreementEprContainer agreementEprContainer = null;
        try
        {
            Query query = em.createNamedQuery( "AgreementEprContainer.findByAgreementId" );
            query.setParameter( "agreementId", agreementId );

            try
            {
                agreementEprContainer = (AgreementEprContainer) query.getSingleResult();
            }
            catch ( NoResultException ex )
            {
                LOG.error( "Could not find an agreement for the agreement id '" + agreementId + "'." );
                throw new Exception( "Could not find an agreement for the EPR address '" + agreementId + "'." );
            }
            catch ( NonUniqueResultException ex )
            {
                LOG.error( "Found multiple agreements for the agreement id '" + agreementId + "'." );
                throw new Exception( "Found multiple agreements for the agreement id '" + agreementId + "'." );
            }
        }
        finally
        {
            em.close();
        }

        // build the agreements' EPR
        return agreementEprContainer.getEpr();
    }
}
