/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.persistence;

import java.text.MessageFormat;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.Query;
import javax.persistence.RollbackException;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.api.AgreementFactory;
import org.ogf.graap.wsag.api.AgreementOffer;
import org.ogf.graap.wsag.api.Negotiation;
import org.ogf.graap.wsag.api.exceptions.AgreementFactoryException;
import org.ogf.graap.wsag.api.exceptions.NegotiationFactoryException;
import org.ogf.graap.wsag.server.persistence.EmfRegistry;
import org.ogf.graap.wsag.server.persistence.PersistedResourceException;
import org.ogf.graap.wsag.server.persistence.PersistentAgreement;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.schemas.graap.wsAgreement.AgreementTemplateType;
import org.ogf.schemas.graap.wsAgreement.negotiation.NegotiationContextType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * The {@link WsDatabaseAgreementFactory} encapsulates a {@link PersistentAgreementFactory} and additionally
 * stores the endpoint reference of the persistent factory. Calls to this factory are delegated to the
 * {@link PersistentAgreementFactory}.
 * 
 * @author thomasw
 * @author owaeld
 */
public class WsDatabaseAgreementFactory
    implements WsPersistentAgreementFactory
{

    private static final Logger LOG = Logger.getLogger( WsDatabaseAgreementFactory.class );

    private final String resourceId;

    private final EndpointReferenceType factoryEpr;

    private final PersistentAgreementFactory pFactory;

    private final List<WsDatabasePersistentAgreement> wsAgreements;

    /**
     * Creates a persistent WS agreement factory for a given EPR and implementation.
     * 
     * @param factory
     *            the persistent factory where calls are delegated to
     * 
     * @param epr
     *            the endpoint reference of the persistent factory
     */
    public WsDatabaseAgreementFactory( PersistentAgreementFactory factory, EndpointReferenceType epr )
    {
        pFactory = factory;
        this.factoryEpr = epr;
        this.resourceId = pFactory.getResourceId();
        this.wsAgreements = new Vector<WsDatabasePersistentAgreement>();
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory#getResourceId()
     */
    public String getResourceId()
    {
        return resourceId;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory#getAgreementFactory()
     * 
     * @deprecated
     */
    @Deprecated
    public AgreementFactory getAgreementFactory()
    {
        return pFactory.getAgreementFactory();
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory#load()
     */
    @SuppressWarnings( "unchecked" )
    public void load() throws Exception
    {
        pFactory.load();

        LOG.debug( "WsSimplePersistentAgreementFactory -> load()" );

        wsAgreements.clear();

        EntityManager em = EmfRegistry.getEntityManager();
        try
        {
            LOG.debug( "WsDatabaseAgreementHome -> list(String agreementFactoryId)" );

            // get a list of all agreement ids with existing EPR
            Query query = em.createNamedQuery( "AgreementEprContainer.findAllByAgreementFactoryId" );
            query.setParameter( "agreementFactoryId", resourceId );

            List<AgreementEprContainer> agreementEprContainers = null;
            try
            {
                agreementEprContainers = query.getResultList();
            }
            catch ( NoResultException ex )
            {
                LOG.error( "Could not find any persisted agreement EPR container." );
            }

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "Found " + agreementEprContainers.size()
                    + " AgreementEprContainer for agreementFactoryId '" + resourceId + "'." );
            }

            // retrieve the agreement for all found EPRs

            for ( AgreementEprContainer agreementEprContainer : agreementEprContainers )
            {
                WsDatabasePersistentAgreement wsDatabasePersistentAgreement =
                    this.findByAgreementId( agreementEprContainer.getAgreementId() );
                wsAgreements.add( wsDatabasePersistentAgreement );
            }

            if ( LOG.isDebugEnabled() )
            {
                LOG.debug( "Loaded " + wsAgreements.size()
                    + " WsDatabasePersistentAgreement(s) for agreementFactoryId '" + resourceId + "'." );
            }
        }
        catch ( Exception ex )
        {
            LOG.error( "Could not load WS database persistent agreements.", ex );
        }
        finally
        {
            // close entity manager
            em.close();
        }

    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory#save()
     */
    public void save() throws Exception
    {
        pFactory.save();
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory#addAgreement(org.ogf.graap.wsag.api.Agreement,
     *      org.w3.x2005.x08.addressing.EndpointReferenceType)
     * 
     * @deprecated
     */
    @Deprecated
    public void addAgreement( Agreement agreement, EndpointReferenceType epr )
    {
        pFactory.addAgreement( agreement, epr );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.AgreementFactory#getTemplates()
     */
    public AgreementTemplateType[] getTemplates()
    {
        return pFactory.getTemplates();
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.AgreementFactory#createAgreement(org.ogf.graap.wsag.api.AgreementOffer)
     */
    public Agreement createAgreement( AgreementOffer offer ) throws AgreementFactoryException
    {
        return pFactory.createAgreement( offer );
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.api.AgreementFactory#initiateNegotiation(NegotiationContextType, XmlObject[],
     *      XmlObject[], Map)
     */
    public Negotiation
        initiateNegotiation( NegotiationContextType context, XmlObject[] criticalExtensions,
                             XmlObject[] nonCriticalExtensions, Map<String, Object> environment )
            throws NegotiationFactoryException
    {

        return pFactory.initiateNegotiation( context, criticalExtensions, nonCriticalExtensions, environment );
    }

    /**
     * Tries to find an agreement for a specified EPR. An agreement can be found, if a corresponding
     * {@link AgreementEprContainer} instance exists.
     * 
     * @param epr
     *            EPR of the agreement.
     * 
     * @return Webservice-version of the persisted agreement.
     * 
     * @throws PersistedResourceException
     *             {@inheritDoc}
     */
    public WsDatabasePersistentAgreement find( String epr ) throws PersistedResourceException
    {

        LOG.debug( "WsDatabaseAgreementHome -> find(String epr)" );

        // try to find the agreement EPR
        EntityManager em = EmfRegistry.getEntityManager();
        Query query = em.createNamedQuery( "AgreementEprContainer.findByEpr" );
        query.setParameter( "epr", epr );

        AgreementEprContainer agreementEprContainer = null;
        try
        {
            agreementEprContainer = (AgreementEprContainer) query.getSingleResult();
        }
        catch ( NoResultException ex )
        {
            LOG.error( "Could not find an agreement for the EPR '" + epr + "'." );
            throw new PersistedResourceException( "Could not find an agreement for the EPR '" + epr + "'." );
        }
        catch ( NonUniqueResultException ex )
        {
            LOG.error( "Found multiple agreements for the EPR '" + epr + "'." );
            throw new PersistedResourceException( "Found multiple agreements for the EPR '" + epr + "'." );
        }

        // build the agreements' EPR
        EndpointReferenceType theEpr = agreementEprContainer.getEpr();

        // build the persistent agreement itself
        PersistentAgreement databasePersistentAgreement =
            pFactory.find( agreementEprContainer.getAgreementId() );
        WsDatabasePersistentAgreement wsDatabasePersistentAgreement =
            new WsDatabasePersistentAgreement( databasePersistentAgreement, theEpr,
                agreementEprContainer.getAgreementFactoryId() );

        // close entity manager
        em.close();

        return wsDatabasePersistentAgreement;
    }

    /**
     * Tries to find a agreement for a specified EPR address of an agreement. An agreement can be found, if a
     * corresponding {@link AgreementEprContainer} instance exists.
     * 
     * @param eprAddress
     *            Address of the EPR of the agreement.
     * 
     * @return Webservice-version of the persisted agreement.
     * 
     * @throws Exception
     *             failed to lookup the EPR
     */
    public WsDatabasePersistentAgreement findByEprAddress( String eprAddress ) throws Exception
    {

        LOG.debug( "WsDatabaseAgreementHome -> findByEprAddress(String eprAddress)" );

        // try to find the agreement EPR
        EntityManager em = EmfRegistry.getEntityManager();
        Query query = em.createNamedQuery( "AgreementEprContainer.findByEprAddress" );
        query.setParameter( "eprAddress", eprAddress );

        AgreementEprContainer agreementEprContainer = null;
        try
        {
            agreementEprContainer = (AgreementEprContainer) query.getSingleResult();
        }
        catch ( NoResultException ex )
        {
            LOG.error( "Could not find an agreement for the EPR address '" + eprAddress + "'." );
            throw new Exception( "Could not find an agreement for the EPR address '\" + epr + \"'." );
        }
        catch ( NonUniqueResultException ex )
        {
            LOG.error( "Found multiple agreements for the EPR address '" + eprAddress + "'." );
            throw new Exception( "Found multiple agreements for the EPR address '" + eprAddress + "'." );
        }

        // build the agreements' EPR
        EndpointReferenceType theEpr = agreementEprContainer.getEpr();

        // build the persistent agreement itself
        PersistentAgreement persistentAgreement = pFactory.find( agreementEprContainer.getAgreementId() );
        WsDatabasePersistentAgreement wsDatabasePersistentAgreement =
            new WsDatabasePersistentAgreement( persistentAgreement, theEpr,
                agreementEprContainer.getAgreementFactoryId() );

        // close entity manager
        em.close();

        return wsDatabasePersistentAgreement;
    }

    /**
     * Creates a list of all webservice-based versions of agreements for one specific agreement factory.
     * 
     * @return a list of webservice-based versions of the agreements for the requested agreement factory.
     */
    public WsDatabasePersistentAgreement[] list()
    {
        return wsAgreements.toArray( new WsDatabasePersistentAgreement[wsAgreements.size()] );
    }

    /**
     * Tries to find a agreement for a specified agreement ID. An agreement can be found, if a corresponding
     * {@link AgreementEprContainer} instance exists.
     * 
     * @param agreementId
     *            ID of the requested agreement.
     * 
     * @return a webservice-version of the persisted agreement.
     * 
     * @throws Exception
     *             failed to look up the specified agreement, i.e. a database error occurred
     */
    public WsDatabasePersistentAgreement findByAgreementId( String agreementId ) throws Exception
    {

        EndpointReferenceType epr = AgreementEprContainer.findEPRForAgreement( agreementId );

        // build the persistent agreement itself
        PersistentAgreement persistentAgreement = pFactory.find( agreementId );
        WsDatabasePersistentAgreement wsDatabasePersistentAgreement =
            new WsDatabasePersistentAgreement( persistentAgreement, epr, getResourceId() );

        return wsDatabasePersistentAgreement;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.persistence.IAgreementHome#list(java.lang.String)
     */
    public PersistentAgreement[] list( String agreementFactoryId ) throws Exception
    {
        if ( resourceId.equals( agreementFactoryId ) )
        {
            return list();
        }
        else
        {
            String message = "Agreement Factory ''{0}'' can not list agreements of factory {1}.";
            LOG.warn( MessageFormat.format( message, new Object[] { resourceId, agreementFactoryId } ) );
            return new PersistentAgreement[0];
        }
    }

    /**
     * Removes the factory identified by the given EPR.
     * 
     * @param epr
     *            the EPR that identifies the factory resource
     * 
     * @throws PersistedResourceException
     *             the specified resource was not removed, i.e. it was not found
     */
    public void remove( String epr ) throws PersistedResourceException
    {

        //
        // we should either use a method like EPR->equals() or the resource Id for matching EPRs
        //
        WsDatabasePersistentAgreement toRemove = null;

        Iterator<WsDatabasePersistentAgreement> it = wsAgreements.iterator();
        while ( it.hasNext() )
        {
            WsDatabasePersistentAgreement current = it.next();
            if ( current.getAgreementEPR().xmlText().equals( epr ) )
            {
                toRemove = current;
                break;
            }
        }

        if ( toRemove == null )
        {
            LOG.error( "Could not find an agreement for the EPR '" + epr + "'." );
            throw new PersistedResourceException( "Could not find an agreement for the EPR '" + epr + "'." );
        }

        // try to find the agreement EPR
        EntityManager em = EmfRegistry.getEntityManager();
        try
        {
            Query query = em.createNamedQuery( "AgreementEprContainer.findByEpr" );
            query.setParameter( "epr", epr );

            // open the transaction
            em.getTransaction().begin();

            AgreementEprContainer agreementEprContainer = null;
            try
            {
                agreementEprContainer = (AgreementEprContainer) query.getSingleResult();
            }
            catch ( NoResultException ex )
            {
                LOG.error( "Could not find an agreement for the EPR '" + epr + "' in database." );
                em.getTransaction().rollback();
                throw new PersistedResourceException( "Could not find an agreement for the EPR '" + epr
                    + "' in database." );
            }
            catch ( NonUniqueResultException ex )
            {
                LOG.error( "Found multiple agreements for the EPR '" + epr + "'." );
                em.getTransaction().rollback();
                throw new PersistedResourceException( "Found multiple agreements for the EPR '" + epr + "'." );
            }

            try
            {
                // remove the EPR for this agreement id and commit
                em.remove( agreementEprContainer );
                em.getTransaction().commit();
                wsAgreements.remove( toRemove );
            }
            catch ( RollbackException ex )
            {
                LOG.error( "Could not remove agreement for EPR '" + epr + "'." );
                em.getTransaction().rollback();
                throw new PersistedResourceException( "Could not remove agreement for EPR '" + epr + "'." );
            }
        }
        finally
        {
            // close entity manager
            em.close();
        }
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.wsrf.persistence.WsPersistentAgreementFactory#getAgreementFactoryEPR()
     */
    public EndpointReferenceType getAgreementFactoryEPR()
    {
        return factoryEpr;
    }

}
