/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.persistence;

import javax.persistence.EntityManager;
import javax.persistence.RollbackException;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.Agreement;
import org.ogf.graap.wsag.server.persistence.EmfRegistry;
import org.ogf.graap.wsag.server.persistence.PersistentAgreement;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * Web service based extension of the
 * {@link org.ogf.graap.wsag.server.persistence.impl.DatabasePersistentAgreement}. It is used to bind an EPR
 * to the agreement, which is stored inside the {@link AgreementEprContainer} instance.
 * 
 * @author T.Weuffel
 */
public class WsDatabasePersistentAgreement implements PersistentAgreement
{

    private final Logger log = Logger.getLogger( WsDatabasePersistentAgreement.class );

    private EndpointReferenceType epr;

    private PersistentAgreement agreement;

    private String factoryId;

    /**
     * Creates a new persisted agreement.
     * 
     * @param agreement
     *            the agreement to persist
     * @param epr
     *            the agreement EPR
     * @param agreementFactoryId
     *            the id of the agreement factory that created the agreement
     * 
     * @see javax.persistence.EntityManagerFactory
     * @see org.ogf.graap.wsag.server.persistence.impl.PersistentAgreementContainer
     */
    public WsDatabasePersistentAgreement( PersistentAgreement agreement, EndpointReferenceType epr,
                                          String agreementFactoryId )
    {
        // set EPR
        this.epr = epr;
        this.agreement = agreement;
        this.factoryId = agreementFactoryId;
    }

    /**
     * {@inheritDoc}
     */
    public void save() throws Exception
    {
        log.warn( "WsDatabasePersistentAgreement -> save()" );

        // try to save the agreements EPR
        EntityManager em = EmfRegistry.getEntityManager();

        // open a transaction
        em.getTransaction().begin();

        AgreementEprContainer agreementEprContainer = null;
        try
        {
            // create a new AgreementEprContainer
            agreementEprContainer = new AgreementEprContainer();
            agreementEprContainer.setAgreementId( agreement.getAgreement().getAgreementId() );
            agreementEprContainer.setAgreementFactoryId( factoryId );
            agreementEprContainer.setEpr( epr );

            // persist the EPR
            em.persist( agreementEprContainer );

            // commit
            em.getTransaction().commit();
        }
        catch ( RollbackException ex )
        {
            log.error( "Could not persist the agreements' EPR. Rollback the commit." );

            em.getTransaction().rollback();

            throw new Exception( "Could not persist the agreements' EPR." );
        }

        // try to save the agreement itself
        try
        {
            agreement.save();
        }
        catch ( Exception ex )
        {
            // persisting the agreement itself was not successful, so remove the EPR
            em.getTransaction().begin();
            em.remove( agreementEprContainer );
            em.getTransaction().commit();
        }

        // close the entity manager
        em.close();
    }

    /**
     * @return the endpoint reference of the web service agreement resource
     */
    public EndpointReferenceType getAgreementEPR()
    {
        return epr;
    }

    /**
     * 
     * @param agreementEpr
     *            the agreement endpoint reference
     */
    public void setAgreementEPR( EndpointReferenceType agreementEpr )
    {
        this.epr = agreementEpr;
    }

    @Override
    public String toString()
    {
        return "WsDatabasePersistentAgreement [agreementId: '" + agreement.getAgreement().getAgreementId()
            + "', name: '" + agreement.getAgreement().getName() + "', agreementFactoryId: '" + factoryId
            + "']";
    }

    /**
     * @return the concrete agreement instance that is associated with the web service resource
     * 
     * @see org.ogf.graap.wsag.server.persistence.PersistentAgreement#getAgreement()
     */
    public Agreement getAgreement()
    {
        return agreement.getAgreement();
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.persistence.PersistentAgreement#load()
     */
    public void load() throws Exception
    {
        agreement.load();
    }
}
