/* 
 * Copyright (c) 2005-2011, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.persistence;

import java.text.MessageFormat;
import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag.server.api.IEngineComponent;
import org.ogf.graap.wsag.server.engine.WsagEngine;
import org.ogf.graap.wsag.server.persistence.IAgreementFactoryHome;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.graap.wsag4j.types.configuration.WSAG4JEngineConfigurationType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3.x2005.x08.addressing.ReferenceParametersType;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * @author thomasw
 */
public class WsDatabaseWSAG4JPersistence implements IEngineComponent, IWsAgreementFactoryHome
{

    private final Logger log = Logger.getLogger( WsDatabaseWSAG4JPersistence.class );

    // default resource-id counter
    private static int counter = 0;

    private final IAgreementFactoryHome agreementFactoryHome;

    private List<WsPersistentAgreementFactory> wsFactories;

    private WSAG4JEngineConfigurationType wsagconfig;

    /**
     * 
     * @param agreementFactoryHome
     *            the home implementation of the agreement factory
     * 
     * @throws Exception
     */
    public WsDatabaseWSAG4JPersistence( IAgreementFactoryHome agreementFactoryHome )
    {
        super();
        this.agreementFactoryHome = agreementFactoryHome;
        wsFactories = new Vector<WsPersistentAgreementFactory>();
    }

    /**
     * {@inheritDoc}
     */
    public void initialize() throws Exception
    {
        wsFactories.clear();

        // delegate method call to agreement factory home instance
        PersistentAgreementFactory[] persistentAgreementFactories = agreementFactoryHome.list();

        // loop over all agreement factories and load the corresponding eprs
        wsFactories = new Vector<WsPersistentAgreementFactory>();

        for ( PersistentAgreementFactory persistentAgreementFactory : persistentAgreementFactories )
        {
            // wrap agreement factory
            WsPersistentAgreementFactory wsPersistentAgreementFactory =
                new WsDatabaseAgreementFactory( persistentAgreementFactory, getNextEPR() );
            wsFactories.add( wsPersistentAgreementFactory );
        }

        //
        // log the initialized factories
        //
        if ( log.isDebugEnabled() )
        {
            log.debug( "Initialize WsDatabaseWSAG4JPersistence instance." );

            WsPersistentAgreementFactory[] wsPersistentAgreementFactories = list();
            log.debug( wsPersistentAgreementFactories.length
                + " WsPersistentAgreementFactory instances available." );

            for ( WsPersistentAgreementFactory wsPersistentAgreementFactory : wsPersistentAgreementFactories )
            {
                WsDatabasePersistentAgreement[] wsDatabasePersistentAgreements =
                    wsPersistentAgreementFactory.list();
                log.debug( "    " + wsDatabasePersistentAgreements.length
                    + " persisted WsDatabasePersistentAgreement instances found." );

                for ( WsDatabasePersistentAgreement wsDatabasePersistentAgreement : wsDatabasePersistentAgreements )
                {
                    log.debug( "        " + wsDatabasePersistentAgreement.toString() );
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public WsPersistentAgreementFactory find( String agreementFactoryId ) throws Exception
    {

        log.debug( "WsDatabaseWSAG4JPersistence -> find(String agreementFactoryId)" );

        // delegate method call to agreement factory home instance
        PersistentAgreementFactory persistentAgreementFactory =
            agreementFactoryHome.find( agreementFactoryId );

        // try to find the corresponding epr
        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();

        // wrap agreement factory
        WsPersistentAgreementFactory wsPersistentAgreementFactory =
            new WsDatabaseAgreementFactory( persistentAgreementFactory, epr );

        return wsPersistentAgreementFactory;
    }

    /**
     * {@inheritDoc}
     */
    public WsPersistentAgreementFactory[] list() throws Exception
    {
        log.debug( "WsDatabaseWSAG4JPersistence -> list()" );

        return wsFactories.toArray( new WsPersistentAgreementFactory[wsFactories.size()] );
    }

    /**
     * {@inheritDoc}
     */
    public void saveAgreementFactories( WsPersistentAgreementFactory[] wsPersistentAgreementFactories )
        throws Exception
    {
        log.debug( "WsDatabaseWSAG4JPersistence -> saveAgreementFactories(WsPersistentAgreementFactory[])" );

        // loop over all ws agreement factories and call the save operation
        for ( WsPersistentAgreementFactory wsPersistentAgreementFactory : wsPersistentAgreementFactories )
        {
            wsPersistentAgreementFactory.save();
        }
    }

    /**
     * {@inheritDoc}
     */
    public void remove( String agreementFactoryId ) throws Exception
    {
        String message = "remove() operation not supported for {0}.";
        throw new UnsupportedOperationException( MessageFormat.format( message,
                                                                       new Object[] { getClass().getName() } ) );
    }

    /**
     * @return the next EPR for a factory resource
     */
    public EndpointReferenceType getNextEPR()
    {
        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        epr.addNewAddress().setStringValue( WsagEngine.getGatewayURL()
                                                + WsagConstants.AGREEMENT_FACTORY_SERVICE_URI );

        ReferenceParametersType ref = epr.addNewReferenceParameters();

        String value = "WSAG4J_ResourceId_" + counter++;

        Document doc = ref.getDomNode().getOwnerDocument();
        Element element =
            doc.createElementNS( WsagConstants.WSAG4J_RESOURCE_ID_QNAME.getNamespaceURI(),
                                 WsagConstants.WSAG4J_RESOURCE_ID_QNAME.getLocalPart() );

        element.appendChild( element.getOwnerDocument().createTextNode( value ) );

        Node imported = doc.importNode( element, true );
        ref.getDomNode().appendChild( imported );

        return epr;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.api.IEngineComponent#setEngineConfiguration(WSAG4JEngineConfigurationType)
     */
    public void setEngineConfiguration( WSAG4JEngineConfigurationType config )
    {
        wsagconfig = config;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.ogf.graap.wsag.server.api.IEngineComponent#getEngineConfiguration()
     */
    public WSAG4JEngineConfigurationType getEngineConfiguration()
    {
        return wsagconfig;
    }
}
