/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.sg.impl;

import org.apache.log4j.Logger;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.ogf.graap.wsag.api.WsagConstants;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3.x2005.x08.addressing.ReferenceParametersType;
import org.w3c.dom.Element;

/**
 * AgreementMembershipContentRule
 * 
 * @author Oliver Waeldrich
 * 
 */
public class AgreementMembershipContentRule extends MembershipContentRule
{

    private static final Logger LOG = Logger.getLogger( AgreementMembershipContentRule.class );

    /**
     * default constructor
     */
    public AgreementMembershipContentRule()
    {
        super();
    }

    /**
     * 
     * @param xml
     *            the rule to set
     * 
     * @see org.apache.muse.ws.resource.sg.impl.SimpleMembershipContentRule
     */
    public AgreementMembershipContentRule( Element xml )
    {
        super( xml );
    }

    /**
     * @param memberEPR
     *            the EPR of the resource that is matched against the membership rule
     * 
     * @return <code>true</code> if the specified resource matches the membership rule, otherwise
     *         <code>false</code>
     */
    public boolean isMatch( EndpointReference memberEPR )
    {
        boolean result = super.isMatch( memberEPR );

        //
        // if this is an agreement resource
        //
        if ( result )
        {
            //
            // The factoryEPR and the registryEPR need to have the same
            // reference parameters in order to have the resource added
            // to this service group.
            //
            EndpointReferenceType factoryEPR = MembershipContentRuleContext.getFactoryEPR();
            EndpointReferenceType registryEPR = MembershipContentRuleContext.getRegistryEPR();

            ReferenceParametersType factoryREF = factoryEPR.getReferenceParameters();
            ReferenceParametersType registryREF = registryEPR.getReferenceParameters();

            //
            // Since the resource to be added is a local wsag4j agreement resource,
            // we simply check the appropriate reference parameter (wsag4j resource id)
            //
            XmlObject[] factoryChildren = factoryREF.selectChildren( WsagConstants.WSAG4J_RESOURCE_ID_QNAME );
            XmlObject[] registryChildren =
                registryREF.selectChildren( WsagConstants.WSAG4J_RESOURCE_ID_QNAME );

            if ( factoryChildren.length != 1 )
            {
                LOG.error( "Expected exacly one occurence of WSAG4J resource id in epr, but found "
                    + Integer.valueOf( factoryChildren.length ) );

                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( factoryEPR.xmlText( new XmlOptions().setSavePrettyPrint() ) );
                }

                return false;
            }

            if ( registryChildren.length != 1 )
            {
                LOG.error( "Expected exacly one occurence of WSAG4J resource id in epr, but found "
                    + Integer.valueOf( registryChildren.length ) );

                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( registryEPR.xmlText( new XmlOptions().setSavePrettyPrint() ) );
                }

                return false;
            }

            String factoryID = factoryChildren[0].getDomNode().getFirstChild().getNodeValue();
            String registryID = registryChildren[0].getDomNode().getFirstChild().getNodeValue();

            if ( factoryID == null )
            {
                LOG.error( "Factory resource id must not be null in reference parameters." );

                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( factoryEPR.xmlText( new XmlOptions().setSavePrettyPrint() ) );
                }

                return false;
            }

            if ( registryID == null )
            {
                LOG.error( "Registry resource id must not be null in reference parameters." );

                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( registryEPR.xmlText( new XmlOptions().setSavePrettyPrint() ) );
                }

                return false;
            }

            result = result && factoryID.equals( registryID );
        }

        return result;
    }

}
