/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.sg.impl;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.metadata.remote.MetadataExchangeClient;
import org.apache.muse.ws.resource.sg.impl.SimpleMembershipContentRule;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;
import org.ogf.graap.wsag.api.WsagConstants;
import org.ogf.graap.wsag4j.types.engine.WSAG4JMetadataDocument;
import org.w3c.dom.Element;

/**
 * MembershipContentRule
 * 
 * @author Oliver Waeldrich
 * 
 */
public class MembershipContentRule extends SimpleMembershipContentRule
{

    private static final Logger LOG = Logger.getLogger( MembershipContentRule.class );

    /**
     * default constructor
     */
    public MembershipContentRule()
    {
        super();
    }

    /**
     * Creates a {@link MembershipContentRule} based on the given XML representaotion of the rule.
     * 
     * @param xml
     *            the rule XML representation
     * 
     * @see SimpleMembershipContentRule
     */
    public MembershipContentRule( Element xml )
    {
        super( xml );
    }

    /**
     * Matches the resource referenced by the given EPR against this rule.
     * 
     * @param memberEPR
     *            the EPR of the resource to match
     * 
     * @return <code>true</code> if the resource matches this rule, <code>false</code> otherwise
     */
    public boolean isMatch( EndpointReference memberEPR )
    {
        boolean result = super.isMatch( memberEPR );

        if ( result )
        {
            QName memberInterface = getMemberInterface();

            if ( memberInterface != null )
            {
                //
                // For Resources that are created locally the port type QName is set
                // in the MembershipContentRuleContext. If not, the resource was not
                // created locally and we must use MEX to lookup the port type.
                //

                QName porttype = MembershipContentRuleContext.getPortTypeQName();

                if ( porttype != null )
                {
                    boolean isMember =
                        memberInterface.getNamespaceURI().equals( porttype.getNamespaceURI() )
                            && memberInterface.getLocalPart().equals( porttype.getLocalPart() );

                    if ( LOG.isDebugEnabled() )
                    {
                        if ( isMember )
                        {
                            LOG.debug( "MembershipContentRule: Resource of type " + porttype.toString()
                                + " is member of service group. (MembershipInterface: "
                                + memberInterface.toString() + "). Resource will be added." );
                        }
                        else
                        {
                            LOG.debug( "MembershipContentRule: Resource of type " + porttype.toString()
                                + " not is member of service group. (MembershipInterface: "
                                + memberInterface.toString() + "). Resource was not added." );
                        }

                    }

                    return isMember;
                }
                else
                {
                    if ( LOG.isDebugEnabled() )
                    {
                        LOG.debug( "MembershipContentRule: "
                            + "Could not retrieve resource port type from MembershipContentRuleContext. "
                            + "Port type is null." );
                    }
                }

                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( "MembershipContentRule: Try to retrieve resource port type WS-MEX." );
                }

                //
                // The resource to be added is a remote resource. We try to lookup the
                // required metadata via MEX.
                //
                try
                {
                    MetadataExchangeClient mex = new MetadataExchangeClient( memberEPR );

                    Element[] response = mex.getMetadata( WsagConstants.WSAG4J_MEX_DIALECT );
                    if ( response.length == 1 )
                    {
                        try
                        {
                            WSAG4JMetadataDocument metadata =
                                (WSAG4JMetadataDocument) XmlObject.Factory.parse( response[0] );
                            porttype = metadata.getWSAG4JMetadata().getWSAG4JPortType();

                            return memberInterface.getNamespaceURI().equals( porttype.getNamespaceURI() )
                                && memberInterface.getLocalPart().equals( porttype.getLocalPart() );

                        }
                        catch ( XmlException e )
                        {
                            //
                            // the returned document is not a valid XML document
                            //
                            return false;
                        }
                        catch ( ClassCastException e )
                        {
                            //
                            // the returned document is not a valid WSAG4JMetadataDocument
                            //
                            return false;
                        }
                    }

                    //
                    // this is not a wsag4j instance
                    //
                    return false;
                }
                catch ( Exception e )
                {
                    //
                    // the MEX service is not configured or reachable
                    //
                    return false;
                }
            }
        }

        return result;
    }

}
