/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.sg.impl;

import javax.xml.namespace.QName;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * MembershipContentRuleContext
 * 
 * A MembershipContentRule may lookup the QName of the port type for resources that are created locally via
 * the MembershipContentRuleContext.
 * 
 * The port type QName is set by the @see ServiceGroup resourceAdded method, and is unset after invocation.
 * 
 * @author Oliver Waeldrich
 * 
 */
public class MembershipContentRuleContext
{

    private static ThreadLocal<RuleContextContainer> context = null;

    /**
     * Initializes the {@link ThreadLocal} during the initialization of the
     * {@link org.ogf.graap.wsag.wsrf.impl.WSAG4JResourceRouter}.
     * 
     * @see org.ogf.graap.wsag.wsrf.impl.WSAG4JResourceRouter#initialize()
     * @see org.ogf.graap.wsag.wsrf.bootstrap.WSAG4JContextListener#contextInitialized(javax.servlet.ServletContextEvent)
     */
    public static synchronized void initializeContext()
    {
        if ( context != null )
        {
            return;
        }

        context = new ThreadLocal<RuleContextContainer>()
        {

            protected RuleContextContainer initialValue()
            {
                return new RuleContextContainer();
            }
        };
    }

    /**
     * Unsets the {@link ThreadLocal} during the shutdown process of the
     * {@link org.ogf.graap.wsag.wsrf.impl.WSAG4JResourceRouter} in order to prevent memory leaks.
     * 
     * @see org.ogf.graap.wsag.wsrf.impl.WSAG4JResourceRouter#shutdown()
     * @see org.ogf.graap.wsag.wsrf.bootstrap.WSAG4JContextListener#contextDestroyed(javax.servlet.ServletContextEvent)
     */
    public static synchronized void finalizeContext()
    {
        //
        // prevent memory leak
        //
        if ( context != null )
        {
            context.set( null );
            context = null;
        }
    }

    /**
     * 
     * @return the QName of the implemented port type
     */
    public static QName getPortTypeQName()
    {
        return context.get().getPortTypeQName();
    }

    /**
     * 
     * @param porttype
     *            the QName of the implemented port type
     */
    public static void setPortTypeQName( QName porttype )
    {
        context.get().setPortTypeQName( porttype );
    }

    /**
     * 
     * @return the agreement factory EPR
     */
    public static EndpointReferenceType getFactoryEPR()
    {
        return context.get().getFactoryEPR();
    }

    /**
     * 
     * @param epr
     *            the agreement factory EPR
     */
    public static void setFactoryEPR( EndpointReferenceType epr )
    {
        ( (RuleContextContainer) context.get() ).setFactoryEPR( epr );
    }

    /**
     * 
     * @return the agreement registry EPR
     */
    public static EndpointReferenceType getRegistryEPR()
    {
        return ( (RuleContextContainer) context.get() ).getRegistryEPR();
    }

    /**
     * 
     * @param epr
     *            the agreement registry EPR
     */
    public static void setRegistryEPR( EndpointReferenceType epr )
    {
        ( (RuleContextContainer) context.get() ).setRegistryEPR( epr );
    }
}
