/* 
 * Copyright (c) 2007, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.wsrf.sg.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.apache.muse.core.Resource;
import org.apache.muse.core.serializer.SerializerRegistry;
import org.apache.muse.ws.addressing.EndpointReference;
import org.apache.muse.ws.addressing.soap.SoapFault;
import org.apache.muse.ws.resource.WsResource;
import org.apache.muse.ws.resource.sg.MembershipContentRule;
import org.apache.muse.ws.resource.sg.ServiceGroupPersistence;
import org.apache.muse.ws.resource.sg.impl.SimpleServiceGroup;

/**
 * WSAG4JServiceGroup
 * 
 * @author Oliver Waeldrich
 * 
 */
public class ServiceGroup extends SimpleServiceGroup
{

    private static final Logger LOG = Logger.getLogger( ServiceGroup.class );

    private Map<EndpointReference, WsResource> entriesByMemberEPR =
        new HashMap<EndpointReference, WsResource>();

    private List<WsResource> entryResources = new Vector<WsResource>();

    /**
     * For local resources we set the port type of the resource in the MembershipContentRuleContext. After
     * invocation we remove it from the context.
     * 
     * @param memberEPR
     *            the endpoint reference of the resource to add to the service group
     * 
     * @param resource
     *            the resource to add to the service group
     * 
     * @throws SoapFault
     *             indicates an error while adding the resource
     * 
     * @see org.apache.muse.ws.resource.sg.impl.SimpleServiceGroup#resourceAdded(org.apache.muse.ws.addressing.EndpointReference
     *      , org.apache.muse.core.Resource)
     */
    public void resourceAdded( EndpointReference memberEPR, Resource resource ) throws SoapFault
    {
        try
        {
            QName porttype = resource.getWsdlPortType();
            if ( porttype != null )
            {
                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( "Created resource with port type " + porttype
                        + ". Setting port type for MembershipContentRuleContext." );
                }

                MembershipContentRuleContext.setPortTypeQName( porttype );
            }
            else
            {
                if ( LOG.isDebugEnabled() )
                {
                    LOG.debug( "Could not set port type for MembershipContentRuleContext. WSDL port type is null." );
                }
            }

            super.resourceAdded( memberEPR, resource );
        }
        finally
        {
            MembershipContentRuleContext.setPortTypeQName( null );
        }
    }

    /**
     * Initializes the service group.
     * 
     * @throws SoapFault
     *             indicates an initialization error
     * 
     * @see org.apache.muse.ws.resource.sg.impl.SimpleServiceGroup#initialize()
     */
    public void initialize() throws SoapFault
    {
        super.initialize();
        SerializerRegistry registry = SerializerRegistry.getInstance();
        registry.registerSerializer( MembershipContentRule.class, new MembershipContentRuleSerializer() );
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.apache.muse.ws.resource.sg.impl.SimpleServiceGroup#resourceRemoved(org.apache.muse.ws.addressing.
     * EndpointReference)
     */
    @Override
    public void resourceRemoved( EndpointReference epr ) throws SoapFault
    {
        //
        // if the SG is destroyed, it means we're being told about
        // our own destruction - don't do anything
        //
        if ( hasBeenShutdown() )
        {
            getWsResource().getResourceManager().removeListener( this );
        }
        else
        {
            WsResource entry = getEntry( epr );

            //
            // if there was no value in the SG for the EPR, it was
            // either a wssg:Entry resource (entries do not have
            // entries themselves, or we'd have infinite recursion)
            // or a member that was removed by a remote client before
            // the member was destroyed
            //
            if ( entry != null )
            {
                entry.shutdown();
            }
        }
    }

    //
    // methods for maintaining the service group
    //

    /**
     * Adds a new entry to the service group.
     * 
     * @param memberEPR
     *            the endpoint reference of the resource to add
     * 
     * @param entry
     *            the resource to add
     * 
     * @return the new service group entry
     */
    public synchronized WsResource addEntry( EndpointReference memberEPR, WsResource entry )
    {
        entriesByMemberEPR.put( memberEPR, entry );
        entryResources.add( entry );
        return entry;
    }

    /**
     * Returns all entries in this service group.
     * 
     * @return the service group entries
     */
    public synchronized WsResource[] getEntry()
    {
        WsResource[] asArray = new WsResource[entryResources.size()];
        return (WsResource[]) entryResources.toArray( asArray );
    }

    /**
     * Retrieves the service group entry identified by the given EPR.
     * 
     * @param memberEPR
     *            the service group entry endpoint reference
     * 
     * @return the service group entry
     */
    public synchronized WsResource getEntry( EndpointReference memberEPR )
    {
        return entriesByMemberEPR.get( memberEPR );
    }

    /**
     * Removes a resource from the service group.
     * 
     * @param entry
     *            the resource to remove
     */
    public synchronized void removeEntry( WsResource entry )
    {
        if ( entry == null )
        {
            throw new NullPointerException( "The entry resource is null." );
        }

        //
        // this is hack-ish - we have to loop through a hash table to find
        // the right resource. don't have time to go back and refactor the
        // way we store entries and make this more efficient - besides, it's
        // more important the additions and lookups be fast
        //
        Iterator<EndpointReference> i = entriesByMemberEPR.keySet().iterator();
        EndpointReference memberEPR = null;

        while ( i.hasNext() && memberEPR == null )
        {
            EndpointReference nextEPR = (EndpointReference) i.next();
            WsResource nextEntry = (WsResource) entriesByMemberEPR.get( nextEPR );

            if ( entry == nextEntry )
            {
                memberEPR = nextEPR;
            }
        }

        String message = "The Entry reference was not found in the ServiceGroup, so it could not be removed.";
        if ( entriesByMemberEPR.remove( memberEPR ) == null )
        {

            throw new RuntimeException( message );
        }

        if ( !entryResources.remove( entry ) )
        {
            throw new RuntimeException( message );
        }

        //
        // if we're using persistence, remove the record of the entry
        //
        ServiceGroupPersistence persistence = (ServiceGroupPersistence) getPersistence();

        try
        {
            if ( persistence != null )
            {
                persistence.resourceRemoved( entry.getEndpointReference() );
            }
        }
        catch ( SoapFault fault )
        {
            LOG.error( fault );
        }
    }

}
